% Example10_3.m
% Program to create video object shape, texture
% and background from a single picture using
% Canny's edge detection followed by the
% morphological operations of closing and opening.

clear
A = imread('clairey040.yuv');
% The Claire image has two stripes at the left and right ends
% and so we want to remove them and keep the rest.
A1 = A(:,11:165);
% Canny's method has two parameters: a) Threshold to block out spurious
% edges, and b) standard deviation which corresponds to the width of the
% Gaussian filter used.
E = edge(A1,'canny',.25,1.25);
E1 = E; % Retain the edge image as a mask
% Use a disk of radius 28 as the structural element for the "imclose"
% function and the edge mask as the structural element for the
% "imopen" function to fill the region within the object boundary. 
E = imopen(imclose(E,strel('disk',28)),E1);
A2 = uint8(zeros(size(A1)));
% To extract the object from the background, assign "0" to the
% region where the edge image is "0" and assign the actual pixel
% values to the region where the edge image is "1".
% The background is obtained by subtracting the object texture 
% from the original picture.
A2(logical(E == 1)) = A1(logical(E == 1));
% Display the various images as a collage.
figure,subplot(2,2,1),imshow(A1),title('Original')
subplot(2,2,2),imshow(E1),title('Shape Boundary')
subplot(2,2,3),imshow(A2),title('Textural Object')
subplot(2,2,4),imshow(double(A1)-double(A2),[]),title('Background')