% Example10_4.m
% Program to calculate vertical and horizontal
% predictions of DC and AC coefficients as per MPEG-4 rule.
% With reference to Fig. 10-21, vertical prediction is used
% if the horizontal gradient is smaller, otherwise, horizontal
% prediction is used. The program calculates the statistics 
% only for the luma component. The block size is 8 x 8 pixels.
% A constant quantization value of 8 is used as an example.

clear
%A1 = imread('cameraman.tif');
A1 = imread('aerial.ras');
% Make image size divisible by 8
[X,Y,Z] = size(A1);
if mod(X,8) ~=0
    Height = floor(X/8)*8;
else
    Height = X;
end
if mod(Y,8) ~=0
    Width = floor(Y/8)*8;
else
    Width = Y;
end
Depth = Z;
clear X Y Z
% if the input image is RGB, then convert it to YCbCr & retain only the 
% Y component.
if Depth == 3
    A1 = rgb2ycbcr(A1(1:Height,1:Width,:));
    A = A1(:,:,1);
else
    A = A1(1:Height,1:Width);
end
% Compute the 8 x 8 DCT of the inpur image
BLKdct = blkproc(double(A),[8 8], @dct2);
% Quantize the DCT coefficients by a constant value of 8.
BLKdctQ = round(BLKdct/8);
% Caculate vertical & horizontal predictions of the
% first column or row of the quantized coefficient block.
% If the horizontal gradient is smaller, then perform
% vertical prediction. Otherwise perform horizontal prediction.
Vcount = 1; Hcount = 1;
for m = 9:8:Height
    for n = 9:8:Width
        gradV = abs(BLKdctQ(m-8,n-8) - BLKdctQ(m-8,n));
        gradH = abs(BLKdctQ(m-8,n-8) - BLKdctQ(m,n-8));
        if gradV > gradH
            Vpredict(Vcount,:) = BLKdctQ(m:m+7,n) - BLKdctQ(m:m+7,n-8);
            Vcount = Vcount + 1;
        else
            Hpredict(:,Hcount) = BLKdctQ(m,n:n+7) - BLKdctQ(m-8,n:n+7);
            Hcount = Hcount + 1;
        end
    end
end
% Compute the histograms of the actual and predicted differentials
Cnt = 1;
for m = 9:8:Height
    for n = 9:8:Width
        Vdct(Cnt,:) = BLKdct(m:m+7,n);
        Hdct(:,Cnt) = BLKdct(m,n:n+7);
        Cnt = Cnt + 1;
    end
end
%
for m = 1:8
    [HorigV(m,:),BorigV(m,:)] = hist(Vdct(:,m),100);
    HorigV(m,:) = HorigV(m,:)/sum(HorigV(m,:));
    [HorigH(m,:),BorigH(m,:)] = hist(Hdct(m,:),100);
    HorigH(m,:) = HorigH(m,:)/sum(HorigH(m,:));
    [Hv(m,:),Bv(m,:)] = hist(Vpredict(:,m),100);
    Hv(m,:) = Hv(m,:)/sum(Hv(m,:));
    [Hh(m,:),Bh(m,:)] = hist(Hpredict(m,:),100);
    Hh(m,:) = Hh(m,:)/sum(Hh(m,:));
end
% Plot the histograms
figure,subplot(2,2,1),plot(BorigV(1,:),HorigV(1,:),'k')
title('Actual DC: Column'),ylabel('Relative Frequency')
subplot(2,2,2),plot(Bv(1,:),Hv(1,:),'k')
title('DC difference: Column'), ylabel('Relative Frequency')
subplot(2,2,3),plot(BorigH(1,:),HorigH(1,:),'k')
title('Actual DC: Row')
xlabel('Coefficient value'), ylabel('Relative Frequency')
subplot(2,2,4),plot(Bh(1,:),Hh(1,:),'k')
title('DC difference: Row')
xlabel('Coefficient value'),ylabel('Relative Frequency')
%
figure,subplot(2,2,1),plot(BorigV(2,:),HorigV(2,:),'k')
title('Actual AC: Column'), ylabel('Relative Frequency')
hold on
for k = 3:8
    plot(BorigV(k,:),HorigV(k,:),'k')
end
hold off
%
subplot(2,2,2),plot(Bv(2,:),Hv(2,:),'k')
title('AC difference: Column'), ylabel('Relative Frequency')
hold on
for k = 3:8
    plot(Bv(k,:),Hv(k,:),'k')
end
hold off
%
subplot(2,2,3),plot(BorigH(2,:),HorigH(2,:),'k')
title('Actual AC: Row')
xlabel('Coefficient value'), ylabel('Relative Frequency')
hold on
for k = 3:8
    plot(BorigH(k,:),HorigH(k,:),'k')
end
hold off
%
subplot(2,2,4),plot(Bh(2,:),Hh(2,:),'k')
title('AC difference: Row')
xlabel('Coefficient value'), ylabel('Relative Frequency')
hold on
for k = 3:8
    plot(Bh(k,:),Hh(k,:),'k')
end
hold off
% 
% calculate the correlation of the DCT coefficient row and column
CorrV = sum(Vdct .* Vdct)/length(Vdct);
for k = 1:8
    CorrH(k) = sum(Hdct(k,:) .* Hdct(k,:))/length(Hdct);
end
figure, subplot(2,1,1), stem(CorrV,'k')
title('Correlation of DCT Column')
ylabel('Correlation')
subplot(2,1,2), stem(CorrH,'k')
title('Correlation of DCT Row')
xlabel('Coefficient #')
ylabel('Correlation')
% Calculate the standard deviations
sprintf('std. dev. vertical prediction: %6.2f\n',std(Vpredict,0,1))
sprintf('std. dev. vertical actual: %6.2f\n',std(Vdct,0,1))
sprintf('std. dev. horizontal prediction: %6.2f\n',std(Hpredict,0,2))
sprintf('std. dev. horizontal actual: %6.2f\n',std(Hdct,0,2))
sprintf('Vertical Corre.: %6.5f\n',CorrV/max(CorrV))
sprintf('Horizontal Corre.: %6.5f\n',CorrH/max(CorrH))
