% Example10_5.m
% Program to do 4 x 4 directional intra prediction
% using the 9 modes specified in H.264.
% The 9 modes are as follows:
% Mode 0: Vertical prediction
% Mode 1: Horizontal prediction
% Mode 2: DC prediction
% Mode 3: Diagonal down-left prediction
% Mode 4: Diagonal down-right prediction
% Mode 5: Vertical-right prediction
% Mode 6: Horizontal-down prediction
% Mode 7: Vertical-left prediction
% Mode 8: Horizontal-up prediction
% Only luma prediction is incorporated.
% No FMO is used and the macro blocks are scanned in
% raster scan order.

clear
A1 = imread('cameraman.tif');
% Make image size divisible by 4
[X,Y,Z] = size(A1);
if mod(X,4) ~=0
    Height = floor(X/4)*4;
else
    Height = X;
end
if mod(Y,4) ~=0
    Width = floor(Y/4)*4;
else
    Width = Y;
end
Depth = Z;
clear X Y Z
% if the input image is RGB, then convert it to YCbCr & retain only the 
% Y component.
if Depth == 3
    A1 = rgb2ycbcr(A1(1:Height,1:Width,:));
    A = double(A1(:,:,1));
else
    A = double(A1(1:Height,1:Width));
end
% Input Y paded by 1 row on the top and
% 1 column to the left and 4 columns to the right.
B = zeros(Height+1,Width+5); 
E = zeros(Height,Width); % Intra prediction error image
% Fill in the B array
B(2:Height+1,2:Width+1) = A;
B(1,2:Width+1) = A(1,:);
B(2:Height+1,1) = A(:,1);
B(1,1) = A(1,1);
for k = 2:5
    B(2:Height+1,Width+k) = A(:,256);
end
for k = 2:5
    B(1,Width+k) = B(2,Width+k);
end
% Do intra 4 x 4 prediction using 9 orientations.
% by calling the function "IntraBlkPredict".
% The returned values are: "PredictedBlk" is the predicted 4 x 4
% block corresponding to the current 4 x 4 block, and
% "Dist" is the SAD value corresponding to the best matching block.
for m = 2:4:Height+1
    for n = 2:4:Width+1
        CurrentBlk = B(m:m+3,n:n+3);
        [PredictedBlk,Dist] = IntraBlkPredict(B,m,n,CurrentBlk);
        E(m-1:m+2,n-1:n+2) = CurrentBlk - PredictedBlk;
    end
end
%
figure,imshow(E,[]), title('Intra prediction error')
%
sprintf('Prediction Gain = %5.2f dB\n',20*log10(std2(A)/std2(E)))
% Compute the prediction error histogram
[H,Bin] = hist(E,100);
figure,plot(Bin,H/sum(H),'k'), title('Intra prediction error histogram')
xlabel('Pixel error')
ylabel('Relative frequency')