function [MatchingBlk,SAD] = IntraBlkPredict(A,Current_row,Current_col,CurrentBlk)
% [MatchingBlk,SAD] = IntraBlkPredict(A,Row,Col,CurrentBlk)
% Function to perform 4 x 4 intra block prediction
% using the 9 directional prediction schemes used in H.264
% Input:
%   A is the paded input image
%   (Current_row, Current_col) is the coordinate of the top left
%   corner of the current block
%   CurrentBlk is the current block being predicted.
% Output:
%   MatchingBlk is the predicted block
%   SAD is the sum of absolute error (city block distance)
%
% The 9 directional prediction modes are numbered 0 through 8.
% The block that results in the least value of SAD is the
% predicted block.
%
SAD = 99999;
MatchingBlk = zeros(8,8);
for k = 0:8
    switch k
        case 0
            % Vertical Mode
            for n = 1:4
                CurrentBlk(:,n) = A(Current_row-1,Current_col+n-1);
            end
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 1
            % Horizontal Mode
            for m = 1:4
                CurrentBlk(m,:) = A(Current_row + m -1,Current_col - 1);
            end
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 2
            % DC (Average) Mode
            Val = sum(A(Current_row-1,Current_col:Current_col+7))...
                + sum(A(Current_row:Current_row+3,Current_col-1))...
                + A(Current_row-1,Current_col-1);
            CurrentBlk(:,:) = round(Val/13);
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 3
            % Diagonal down left Mode
            for m = 1:4
                n = m + 3;
                CurrentBlk(m,1:4) = A(Current_row -1,Current_col+m:Current_col+n);
            end
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 4
            % Diagonal down right Mode
            for m = 1:4
                CurrentBlk(m,m) = A(Current_row -1,Current_col-1);
            end
            for m = 2:4
                CurrentBlk(m,m-1) = A(Current_row,Current_col-1);
            end
            for m = 3:4
                CurrentBlk(m,m-2) = A(Current_row+1,Current_col-1);
            end
            CurrentBlk(4,1) = A(Current_row+2,Current_col-1);
            for n = 2:4
                CurrentBlk(n-1,n) = A(Current_row-1,Current_col);
            end
            for n = 3:4
                CurrentBlk(n-2,n) = A(Current_row-1,Current_col+1);
            end
            CurrentBlk(1,4) = A(Current_row-1,Current_col+2);
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 5
            % Vertical right Mode
            for m = 1:4
                CurrentBlk(m,m) = A(Current_row-1,Current_col);
            end
            for m = 1:3
                CurrentBlk(m,m+1) = A(Current_row-1,Current_col+1);
            end
            for m = 1:2
                CurrentBlk(m,m+2) = A(Current_row-1,Current_col+2);
            end
            CurrentBlk(1,4) = A(Current_row-1,Current_col+3);
            for m = 2:4
                CurrentBlk(m,m-1) = A(Current_row-1,Current_col-1);
            end
            for m = 3:4
                CurrentBlk(m,m-2) = A(Current_row,Current_col-1);
            end
            CurrentBlk(4,1) = A(Current_row+1,Current_col-1);
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 6
            % horizontal down Mode
            for m = 1:4
                CurrentBlk(m,m) = A(Current_row,Current_col-1);
            end
            for m = 2:4
                CurrentBlk(m,m-1) = A(Current_row+1,Current_col-1);
            end
            for m = 3:4
                CurrentBlk(m,m-2) = A(Current_row+2,Current_col-1);
            end
            CurrentBlk(4,1) = A(Current_row+3,Current_col-1);
            for m = 1:3
                CurrentBlk(m,m+1) = A(Current_row-1,Current_col-1);
            end
            for m = 1:2
                CurrentBlk(m,m+2) = A(Current_row-1,Current_col);
            end
            CurrentBlk(1,4) = A(Current_row-1,Current_col+1);
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 7
            % Vertcal left Mode
            CurrentBlk(1,1) = A(Current_row-1,Current_col);
            CurrentBlk(1,2) = A(Current_row-1,Current_col+1);
            CurrentBlk(2,1) = A(Current_row-1,Current_col+1);
            CurrentBlk(3,1) = A(Current_row-1,Current_col+1);
            CurrentBlk(1,3) = A(Current_row-1,Current_col+2);
            CurrentBlk(2,2) = A(Current_row-1,Current_col+2);
            CurrentBlk(3,2) = A(Current_row-1,Current_col+2);
            CurrentBlk(4,1) = A(Current_row-1,Current_col+2);
            CurrentBlk(1,4) = A(Current_row-1,Current_col+3);
            CurrentBlk(2,3) = A(Current_row-1,Current_col+3);
            CurrentBlk(3,3) = A(Current_row-1,Current_col+3);
            CurrentBlk(4,2) = A(Current_row-1,Current_col+3);
            CurrentBlk(2,4) = A(Current_row-1,Current_col+4);
            CurrentBlk(3,4) = A(Current_row-1,Current_col+4);
            CurrentBlk(4,3) = A(Current_row-1,Current_col+4);
            CurrentBlk(4,4) = A(Current_row-1,Current_col+5);
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
        case 8
            % horizontal up Mode
            CurrentBlk(1,1) = A(Current_row,Current_col-1);
            CurrentBlk(2,1) = A(Current_row+1,Current_col-1);
            CurrentBlk(1,2) = A(Current_row+1,Current_col-1);
            CurrentBlk(1,3) = A(Current_row+1,Current_col-1);
            CurrentBlk(3,1) = A(Current_row+2,Current_col-1);
            CurrentBlk(2,2) = A(Current_row+2,Current_col-1);
            CurrentBlk(2,3) = A(Current_row+2,Current_col-1);
            CurrentBlk(1,4) = A(Current_row+2,Current_col-1);
            CurrentBlk(4,1) = A(Current_row+3,Current_col-1);
            CurrentBlk(3,2) = A(Current_row+3,Current_col-1);
            CurrentBlk(3,3) = A(Current_row+3,Current_col-1);
            CurrentBlk(2,4) = A(Current_row+3,Current_col-1);
            CurrentBlk(4,2) = A(Current_row+3,Current_col-1);
            CurrentBlk(4,3) = A(Current_row+3,Current_col-1);
            CurrentBlk(3,4) = A(Current_row+3,Current_col-1);
            CurrentBlk(4,4) = A(Current_row+3,Current_col-1);
            d = sum(sum(abs(CurrentBlk-A(Current_row:Current_row+3,Current_col:Current_col+3))));
            if d < SAD
                SAD = d;
                MatchingBlk = CurrentBlk;
            end
    end
end
    