% Dithering.m
% Quantize an input image using a uniform quantizer
% with dithering to eliminate contouring effect
% Dither is a uniformly distributed pseudo random noise
% between [-8,8]
clear
close all
%
B = 4; % # bits of quantization
A = imread('barbara.tif');
%A = imread('cameraman.tif');
[Height,Width,Depth] = size(A);
if Depth == 1
    f = double(A);
else
    f = double(A(:,:,1));
end
if Height>512 || Width > 512
    figure,imshow(uint8(f(60:275,302:585)))
    [Counts,Bins] = imhist(uint8(f(60:275,302:585)),256);
else
    figure,imshow(uint8(f))
    [Counts,Bins] = imhist(uint8(f),256);
end
title('Image quantized to 8 bits')
%
u = 16*(rand(Height,Width)-0.5); % dither or pseudo random noise
f1 = f + u; % noise added to input image
f2 = zeros(size(f));% quantized image without dithering
g = zeros(size(f)); % quantized image with dithering
g1 = zeros(size(f)); % qutized image with dithering with no subtraction
% after quantization
% Design a uniform quantizer with B bpp
L = 2^B;
q = 255/L;
q2 = q/2;
d = linspace(0,255,L+1); % decision regions
r = zeros(L,1); % output levels
for k = 1:L
    r(k) = d(k)+q2;
end
% Quantize input image without dithering
for k = 1:L
    [x,y] = find(f>d(k) & f<=d(k+1));
    for j = 1:length(x)
        f2(x(j),y(j)) = round(r(k));
    end
end
% Quantize input image with dithering
for k = 1:L
    [x,y] = find(f1>d(k) & f1<=d(k+1));
    for j = 1:length(x)
        g(x(j),y(j)) = round(r(k))-u(x(j),y(j));
        g1(x(j),y(j)) = round(r(k));
    end
end
% display quantized image without dithering
if Height>512 || Width > 512
    figure,imshow(uint8(f2(60:275,302:585)))
else
    figure,imshow(uint8(f2))
end
title(['No dithering: Image quantized to ' num2str(B) ' bits'])
% display quantized image with dithering
if Height>512 || Width > 512
    figure,imshow(uint8(g(60:275,302:585)))
else
    figure,imshow(uint8(g))
end
title(['With dithering: Image quantized to ' num2str(B) ' bits'])
if Height>512 || Width > 512
    figure,imshow(uint8(g1(60:275,302:585)))
else
    figure,imshow(uint8(g1))
end
title(['No dither subtraction: Image quantized to ' num2str(B) ' bits'])