% Example2_2.m
% Example showing the effect of image sampling with a
% finite width pulse
% Two cases are possible: 'delta' and 'pulse'
% delta corresponds to impulse sampling and pulse
% to sampling with finite width pulse.
% image array size is NxN pixels
% actual image area to be sampled is MxM
% TxT is the sampling pulse area

samplingType = 'pulse'; % 'delta' or 'pulse'
N = 256; % array size is NxN
f = zeros(N,N); % image to be sampled
M = 32; % image pulse width
M1 = N/2 - M + 1; % image beginning point
M2 = N/2+M; % image end point
f(M1:M2,M1:M2) = 1; % test image has a value 1
%
p = zeros(N,N); % sampling rectangular pulse
if strcmpi(samplingType,'delta')
    p(128,128) = 1;
end
if strcmpi(samplingType,'pulse')
    T = M/2; % sampling pulse width
    T1 = N/2 - T + 1; % sampling pulse beginning point
    T2 = N/2+T; % sampling pulse end point
    p(T1:T2,T1:T2) = 1; % sampling pulse; width = 1/2 of image
end
fs = conv2(f,p,'same'); % convolution of image & pulse
figure,imshow(f,[]),title('Input Image')
figure,imshow(p,[]), title('Sampling Pulse')
figure,imshow(fs,[]),title('Sampled Image')
%
figure,imshow(log10(1+5.5*abs(fftshift(fft2(f)))),[])
title('Fourier Transform of input image')
if strcmpi(samplingType,'delta')
    figure,imshow(log10(1+1.05*abs(fftshift(fft2(p)))),[])
    title('Fourier Transform of Delta Function')
end
if strcmpi(samplingType,'pulse')
    figure,imshow(log10(1+5.5*abs(fftshift(fft2(p)))),[])
    title('Fourier Transform of rectangular pulse')
end
%
Fs = fftshift(fft2(fs));
figure,imshow(log10(1+5.5*abs(Fs)),[])
title('Fourier Transform of sampled image')
% h is the 2D reconstruction filter
%h = (sinc(0.1*pi*(-128:127)))' * sinc(0.1*pi*(-128:127));
h = (sinc(0.2*pi*(-128:127)))' * sinc(0.2*pi*(-128:127));
H = fft2(h);
figure,imshow(ifftshift(ifft2(fft2(fs).*H,'symmetric')),[])
title('Reconstructed image')