% Example2_5.m
% Uniform quantizer design
% Computes decision intervals and output levels
% for quantization bits from 1 to B
% Quantizes the input image to B bits and
% Computes Signal-to-Quantization noise in dB
% for 1 to B bits and plots 
%
clear
close all
%
A = imread('barbara.tif');
%A = imread('cameraman.tif');
[Height,Width,Depth] = size(A);
if Depth == 1
    f = double(A);
else
    f = double(A(:,:,1));
end
if Height>512 || Width > 512
    figure,imshow(uint8(f(60:275,302:585)))
    [Counts,Bins] = imhist(uint8(f(60:275,302:585)),256);
else
    figure,imshow(uint8(f))
    [Counts,Bins] = imhist(uint8(f),256);
end
title('Image quantized to 8 bits')
% Calculate the histogram of the input image
figure,plot(Bins,Counts,'k', 'LineWidth', 2)
title('Histogram of input image')
xlabel('Pixel Value')
ylabel('Count')
%
f1 = zeros(size(f));
E = zeros(size(f));
sigmaf = std2(f);
B = 7;
snr = zeros(B,1);
% Quantization method 1 is the process of using decision intervals
% and corresponding reconstruction levels
% method 2 is similar to JPEG quantization rule
QuantizerMethod = 1;% options 1 or 2
switch QuantizerMethod
    case 1
        % Design a uniform quantizer for bits 1 to 7
        for b = 1:B
            L = 2^b;
            q = 255/L;
            q2 = q/2;
            d = linspace(0,255,L+1);
            r = zeros(L,1);
            for k = 1:L
                r(k) = d(k)+q2;
            end
            for k = 1:L
                [x,y] = find(f>d(k) & f<=d(k+1));
                for j = 1:length(x)
                    f1(x(j),y(j)) = round(r(k));
                    E(x(j),y(j)) = double(f(x(j),y(j)))-r(k);% include surf plot
                end
                if b == 5 && k == 32
                    figure,subplot(2,1,1),plot(E(150,:),'k','LineWidth',2)
                    title(['Quantization error: L = ' num2str(k)])
                    xlabel('pixel # on row 150')
                    ylabel('Quantization error')
                    [Counts,Bins]= hist(E(150,:));
                    subplot(2,1,2),plot(Bins,Counts,'k','LineWidth',2)
                    title('Histogram of quantization error')
                    xlabel('Error bins'), ylabel('Counts')
                end
                clear x y
            end
            clear E
            if b == 3 || b == 5
                if Height>512 || Width > 512
                    figure,imshow(uint8(f1(60:275,302:585)))
                else
                    figure,imshow(uint8(f1))
                end
                title(['Image quantized to ' num2str(b) ' bits'])
            end
            snr(b) = 20*log10(sigmaf/std2(f-f1));
        end
        figure,plot(1:B,snr,'k','LineWidth',2)
        title('SNR Vs Bits/pixel of a uniform quantizer')
        xlabel('# bits')
        ylabel('SNR(dB)')
        figure,stairs(d(1:L),r,'k','LineWidth',2)
        title('Uniform Quantizer: Decision intervals & output levels')
        xlabel('Decision Regions')
        ylabel('Output Levels')
    case 2
        for b = 1:B
            L = 2^b;
            q = 255/L;
            f1 = round(floor(f/q + 0.5)*q);
            if b == 5
                E = f - f1;
                figure,subplot(2,1,1),plot(E(150,:),'k','LineWidth',2)
                title(['Quantization error: L = ' num2str(L)])
                xlabel('pixel # on row 150')
                ylabel('Quantization error')
                [Counts,Bins]= hist(E(150,:));
                subplot(2,1,2),plot(Bins,Counts,'k','LineWidth',2)
                title('Histogram of quantization error')
                xlabel('Error bins'), ylabel('Counts')
            end
            if b == 3 || b == 5
                if Height>512 || Width > 512
                    figure,imshow(uint8(f1(60:275,302:585)))
                else
                    figure,imshow(uint8(f1))
                end
                title(['Image quantized to ' num2str(b) ' bits'])
            end
            snr(b) = 20*log10(sigmaf/std2(f-f1));
        end
        figure,plot(1:B,snr,'k','LineWidth',2)
        title('SNR Vs Bits/pixel of a uniform quantizer')
        xlabel('# bits')
        ylabel('SNR(dB)')
end