% Example2_6.m
% Design of a Nonuniform Quantizer for a given input image
% Starts with an initial set of output levels and
% iteratively builds a final set that results in the
% least mean square error between the input and
% reconstructed image
clear
close all

%
%A = imread('cameraman.tif');
%A = imread('aerial.ras');
%A = imread('barbara.tif');
%A = imread('lighthouse.ras');
%A = imread('birds.ras');
%A = imread('masuda2.ras');
%A = imread('airplane.ras');
%A = imread('yacht.ras');
%A = imread('hatwoman.tif');
%A = imread('peppers.png');
%A = imread('autumn.tif');
A = imread('bicycles.ras');
%A = imread('girl.ras');
%A = imread('kids.ras');
[Height,Width,Depth] = size(A);
if Depth == 1
    f = double(A);
else
    f = double(rgb2gray(A));
end
%
figure,imshow(f,[]);
title('Original 8 bpp image')
%
sigmaf = std2(f); % std dev of input image
B = 7; % Maximum number of bits of quantization
snr = zeros(B,1); % signal-to-quantization noise in dB
NumIteration = zeros(B,1); % # iterations
err = 0.1e-01; % accuracy to terminate iteration
% Compute iteratively L output levels
for b = 1:B
    L = 2^b; % number of output levels
    % initial output levels
    C = linspace(round(255/(2*L)),255-round(255/(2*L)),L);
    PartitionRegion = zeros(L,Height,Width); % partitions consisting of input pixels
    RegionCount = zeros(L,1); % number of elements in each partition
    RegionSum = zeros(L,1); % sum of pixels in a given partition region
    Distortion = zeros(L,1); % minimum distortion in a partition region
    TotalDistortion = 1.0e+5; % total minimum distortion
    %TD = abs(TotalDistortion-sum(Distortion)); % initial total distortion
    % alternate convergence measure
    TD = (TotalDistortion-sum(Distortion))/TotalDistortion;
    % start iteration
    It = 1; % iteration number
    while TD >= err
        for r = 1:Height
            for c = 1:Width
                d1 = abs(f(r,c)-C(1));
                J = 1;
                for k = 2:L
                    d2 = abs(f(r,c)-C(k));
                    if d2<d1
                        d1 = d2;
                        J = k;
                    end
                end
                RegionCount(J) = RegionCount(J) + 1;
                RegionSum(J) = RegionSum(J) + f(r,c);
                PartitionRegion(J,r,c) = f(r,c);
                Distortion(J) = Distortion(J)+(f(r,c)-C(J))*(f(r,c)-C(J));
            end
        end
        Index = logical(RegionCount == 0);% check for empty regions
        RegionCount(Index) = 1;% set empty regions to count 1
        Distortion = Distortion ./RegionCount;% average minimum distortions
        %TD = abs(TotalDistortion-sum(Distortion));
        % alternate convergence measure
        TD = (TotalDistortion-sum(Distortion))/TotalDistortion;
        C = RegionSum ./RegionCount; % refined output levels
        TotalDistortion = sum(Distortion);% total minimum distortion
        It = It + 1;
    end % End iteration
    NumIteration(b) = It;
    clear Index
    sprintf('Number of iterations = %d; \t bpp = %d',It-1,b)
    %sprintf('SNR = %4.2f dB',10*log10(sigmaf*sigmaf/TotalDistortion))
    % make sure that C(k) >= C(k-1), 1 <= k <= L
    for k = 1:L-1
        if C(k+1) < C(k)
            C(k+1) = C(k);
        end
    end
    % Compute the input decision regions from output levels
    DR = zeros(L+1,1);
    DR(L+1) = 255;
    f1 = zeros(size(f));
    for k = 2:L
        DR(k) = (C(k-1)+C(k))/2;
    end
    % plot output levels against decision regions
    if b == 5
        figure,stairs(DR(1:L),C,'k','LineWidth',2)
        title(['Nonuniform quantizer: ' num2str(b) ' bpp'])
        xlabel('Decision Regions')
        ylabel('Output Levels')
        figure,subplot(2,1,1),stem(RegionCount/(It-1),'k')
        title(['Comparison of output & input histograms at ' num2str(b) ' bpp'])
        ylabel('Counts')
        [Counts,Bins] = imhist(uint8(f),32);
        subplot(2,1,2),stem(Counts,'k')
        xlabel('Level number'), ylabel('Counts')
    end
    % decode using the codebook generated
    %{
    for r = 1:Height
        for c = 1:Width
            d1 = abs(f(r,c)-C(1));
            for k = 2:L
                d2 = abs(f(r,c)-C(k));
                if d2<d1
                    d1 = d2;
                    J = k;
                end
            end
            f1(r,c) = C(J);
        end
    end
 %}
    % decode using decision regions & codebook: This is better than
    % the other decoding scheme.
    %
    for k = 1:L
        [x,y] = find(f>DR(k) & f<=DR(k+1));
        if ~isempty(x)
            for j = 1:length(x)
                f1(x(j),y(j)) = C(k);
            end
        end
        clear x y
    end
    %
    if b == 3 || b == 5
        figure,imshow(uint8(f1)) % disply requantized image
        title(['Requantized image at ' num2str(b) ' bpp'])
    end
    mse = sum(sum((f-f1).*(f-f1)))/(Height*Width);
    snr(b) = 10*log10(sigmaf*sigmaf/mse);
end
%
figure,plot(1:B,snr,'k','LineWidth',2)
title('SNR Vs Bits/pixel of a nonuniform quantizer')
xlabel('# bits')
ylabel('SNR(dB)')
%
figure,plot(1:length(NumIteration),NumIteration,...
    'k','LineWidth',2)
title('# iterations Vs No. bits')
xlabel('# bits'), ylabel('# iterations')
text(3,0.9*max(NumIteration),['Convergence error = ' num2str(err)])
