% BasisRestriction.m
% Exapand a given image in terms of the
% basis images corresponding to a user
% defined 2D unitary transform and size
%
N = 8;
% options: 'DCT', 'DST', 'Hartley', 'Haar', 
% 'Hadamard' and 'Slant'
%
% calculate the 1D transform kernel and the basis images
% by calling the function BasisImages
%
% read an image
A = imread('barbara.tif');
Height = 256;
Width = 256;
x = A(59:58+Height,331:330+Width,1);% crpped input image
%{
% alternative image to read
A = imread('cameraman.tif');
%[Height,Width,Depth] = size(A);
x = A(:,:,1);
%}
clear A
%
% reconstructed image
% mse corresponding to using 1 to N*N basis images
% expand input image in terms of basis images and 
% calculate mse due to using 1 to N*N basis images in reconstruction
for k = 1:6
    if k == 1
        TransformType = 'DCT';
        [y,T] = BasisImages(N,TransformType);
    elseif k == 2
        TransformType = 'Hartley';
        [y,T] = BasisImages(N,TransformType);
    elseif k == 3
        TransformType = 'Haar';
        [y,T] = BasisImages(N,TransformType);
    elseif k == 4
        TransformType = 'Hadamard';
        [y,T] = BasisImages(N,TransformType);
    elseif k == 5
        TransformType = 'Slant';
        [y,T] = BasisImages(N,TransformType);
    else
        TransformType = 'KLT';
        [y1,y2,T] = KLTbasisImages(N,double(x));
    end
    x2 = zeros(size(x));
    mse = zeros(N,N);
    %{
    for r = 1:N:Height % take N rows at a time
        for c = 1:N:Width   % take N columns at a time
            x1 = y*double( x(r:r+N-1,c:c+N-1))*y';
            CoefMean = CoefMean + x1;
            x2(r:r+N-1,c:c+N-1) = x1;
        end
    end
    CoefMean = CoefMean/(Height*Width/(N*N));
    for r = 1:N:Height % take N rows at a time
        for c = 1:N:Width   % take N columns at a time
            CoefVar = CoefVar + (x2(r:r+N-1,c:c+N-1)-CoefMean).*(x2(r:r+N-1,c:c+N-1)-CoefMean);
        end
    end
    CoefVar = CoefVar/(Height*Width/(N*N));
    if k == 1
        figure, plot(0:N^2-1,[CoefVar(1,:) CoefVar(2,:) CoefVar(3,:) CoefVar(4,:) CoefVar(5,:) CoefVar(6,:)...
            CoefVar(7,:) CoefVar(8,:)],'k:','LineWidth',2)
        hold on
    else
        plot(0:N^2-1,[CoefVar(1,:) CoefVar(2,:) CoefVar(3,:) CoefVar(4,:) CoefVar(5,:) CoefVar(6,:)...
            CoefVar(7,:) CoefVar(8,:)],'k:','LineWidth',2)
    end
    %}
    for m = 0:N-1   % take each basis image along rows
    mInd = m*N + 1;
        for n = 0:N-1   % take each basis image along columns
            nInd = n*N + 1;
            for r = 1:N:Height % take N rows at a time
                for c = 1:N:Width   % take N columns at a time
                    if k == 6
                        x1 = y1'*double( x(r:r+N-1,c:c+N-1))*y2;
                    else
                        x1 = y*double( x(r:r+N-1,c:c+N-1))*y';
                    end
                    sum1 = x2(r:r+N-1,c:c+N-1);% accumulate image
                    x2(r:r+N-1,c:c+N-1) = sum1 +...
                    x1(m+1,n+1) * T(mInd:mInd+N-1,nInd:nInd+N-1);
                end
            end
            mse(m+1,n+1) = (std2(double(x) - x2)) ^2;
        end
    end
    if k == 1
        figure,plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
        mse(7,:) mse(8,:)],'k:','LineWidth',2)
        hold on
    elseif k == 2
        plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
        mse(7,:) mse(8,:)],'ko','LineWidth',1)
    elseif k == 3
        plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
        mse(7,:) mse(8,:)],'k+','LineWidth',1)
    elseif k == 4
        plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
        mse(7,:) mse(8,:)],'ks','LineWidth',1)
    elseif k == 5
        plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
        mse(7,:) mse(8,:)],'k*','LineWidth',1)
    else
        plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
        mse(7,:) mse(8,:)],'k','LineWidth',2)
    end
end
xlabel(' # of basis images used')
ylabel('MSE')
hold off
legend('DCT','Hartley','Haar','Hadamard','Slant', 'KLT',0)