% Example3_4.m
% Exapand a given image in terms of the
% basis images corresponding to a user
% defined 2D unitary transform and size
%
N = 8;
TransformType = 'DCT';% options: 'DCT', 'DST', 'Hartley', 'Haar', 
% 'Hadamard' and 'Slant'
%
% calculate the 1D transform kernel and the basis images
% by calling the function BasisImages
[y,T] = BasisImages(N,TransformType);
% read an image
A = imread('barbara.tif');
Height = 256;
Width = 256;
x = A(59:58+Height,331:330+Width,1);% crpped input image
%{
% alternative image to read
A = imread('cameraman.tif');
%[Height,Width,Depth] = size(A);
x = A(:,:,1);
%}
clear A
%
x1 = zeros(N,N);% N x N array of transform coefficients
x2 = zeros(size(x)); % reconstructed image
mse = zeros(N,N); % mse corresponding to using 1 to N*N basis images
% expand input image in terms of basis images and 
% calculate mse due to using 1 to N*N basis images in reconstruction
for m = 0:N-1   % take each basis image along rows
    mInd = m*N + 1;
    for n = 0:N-1   % take each basis image along columns
        nInd = n*N + 1;
        for r = 1:N:Height % take N rows at a time
            for c = 1:N:Width   % take N columns at a time
                x1 = y*double( x(r:r+N-1,c:c+N-1))*y';
                sum1 = x2(r:r+N-1,c:c+N-1);% accumulate image
                x2(r:r+N-1,c:c+N-1) = sum1 +...
                    x1(m+1,n+1) * T(mInd:mInd+N-1,nInd:nInd+N-1);
            end
        end
        % show reconstructed image with 1 and 13 basis images
        if (m == 0 && n == 0) || (m == 1 && n == 4)
            figure,imshow(x2,[])
            title(['No. of basis images used = ' num2str(m*N+n+1) ])
        end
        mse(m+1,n+1) = (std2(double(x) - x2)) ^2;
    end
end 
%
figure,imshow(x2,[])
title('Reconstruction using all basis images')
%
figure,plot(0:N^2-1,[mse(1,:) mse(2,:) mse(3,:) mse(4,:) mse(5,:) mse(6,:)...
    mse(7,:) mse(8,:)],'k:','LineWidth',2)
xlabel([' # of ' TransformType ' basis images used'])
ylabel('MSE'), title(['2D ' TransformType ' Transform']);