% Example3_6.m
% computes the autocorrelation matrix of
% transformed image using unitary transforms
% Unitary transform types are: DCT, DST, Hartley,
% Haar, Hadamard, Slant, and KL
N = 8; % N x N is the transform size
%
% Barbara image is fairly large and so is cropped to size 256 x 256 pixels.
A = imread('barbara.tif');
Height = 256;
Width = 256;
x = double(A(59:58+Height,331:330+Width,1));% cropped image
R = AutoCorrelation(x,N); % call AutoCorrelation to compute Rxx
%
% Next, calculate the N x N unitary transform and then
% compute the autocorrelation matrix of the transformed image
% using Ryy = y*Rxx*y' for all except KL and
% Ryy = y'*Rxx*y for KL transform.
for k = 1:7
    if k == 1
        TransformType = 'DCT';
    elseif k == 2
        TransformType = 'DST';
    elseif k == 3
        TransformType = 'Hartley';
    elseif k == 4
        TransformType = 'Haar';  
    elseif k == 5
        TransformType = 'Hadamard';
    elseif k == 6
        TransformType = 'Slant';
    else
        TransformType = 'KLT';
    end
    if k <=6
        % for all except KL
        y = UnitaryTransform(N,TransformType);
        Ry = y*R*y';
    else
        [y,Evalc] = eig(R);% eigen matrix of R
        % reorder the covariance matrix in descending order of eigen values
        y = y(:,N:-1:1);
        y = y(N:-1:1,:);
        Ry = y'*R*y;
    end
    %
    Ry = abs(Ry);
    % plot the absolute of Ryy for each transform with different color
    % and line style.
    if k == 1
        figure,semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'k','LineWidth',2)
        hold on
    elseif k == 2
        semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'g:','LineWidth',1)
    elseif k == 3
        semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'r-.','LineWidth',1)
    elseif k == 4
        semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'b--','LineWidth',1)
    elseif k == 5
        semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'m:','LineWidth',1)
    elseif k == 6
        semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'c--','LineWidth',1)
    else
        semilogy(0:N^2-1,[Ry(1,:) Ry(2,:) Ry(3,:) Ry(4,:) Ry(5,:) Ry(6,:)...
            Ry(7,:) Ry(8,:)],'k','LineWidth',1)
    end
end
legend('DCT','DST','Hartley','Haar','Hadamard','Slant', 'KLT',0)
xlabel('(I,J)')
ylabel('abs(R)')
hold off