function y = FastDCT(x)
% y = FastDCT(x)
% computes the dct of an N x N block x
% using decimation in time FFT algorithm
% x is an N x N matrix and y is the dct of x
% N is assumed to be of the type 2^m, m is a
% positive integer

N = size(x,1);
x1 = zeros(N,N);
y = zeros(N,N);
% obtain x1 from x: x1(n) = x(2n), 0 <= n <= N/2;
% x1(N-1-n)= x(2n+1), 0<= n <= N/2
x1(:,1:N/2) = x(:,1:2:N);
x1(:,N+1-(1:N/2))= x(:,2:2:N);
% compute the DFT of each row of x1: X1(k) = sum(x1(n)*WN^nk);
for m = 1:N
    x1(m,:) = fft(x1(m,:));
end
%multiply DFT by alpha(k) and W2N^(k/2) and take the real part
% alpha(0) = 1/sqrt(N) and alpha(2:N) = sqrt(2/N);
% W2N = exp(-i*2*pi/(2*N))
% MATLAB executes complex function faster if you use 1i instead of i
% where i = sqrt(-1).
a(1) = 1/sqrt(N);
a(2:N) = sqrt(2/N);
W2N = exp(-1i*2*pi/(2*N));
for m = 1:N
    y(m,1:N) = real(a.*W2N .^((0:N-1)/2).*x1(m,1:N));
end
% Now compute the DCT along columns which is the same
% as computing the DCT of y' along rows
y = y';
x1(:,1:N/2) = y(:,1:2:N);
x1(:,N+1-(1:N/2))= y(:,2:2:N);
for m = 1:N
    x1(m,:) = fft(x1(m,:));
end
% Agaian, multiply the dft by alpha(k) and W2N^(k/2)
for m = 1:N
    y(m,1:N) = real(a.*W2N .^((0:N-1)/2).*x1(m,1:N));
end
% transpose y to get the dct in the same order as input pixel block x
y = y';






