function y = FastDCT(x)
% y = FastDCT(x)
% computes the dct of an N-point sequence x
% using decimation in time FFT algorithm
% x is an 1 x N vector and y is the dct of x

N = length(x);
x1 = zeros(1,N);
% obtain x1 from x: x1(n) = x(2n), 0 <= n <= N/2;
% x1(N-1-n)= x(2n+1), 0<= n <= N/2
x1(1,1:N/2) = x(1,1:2:N);
x1(1,N+1-(1:N/2))= x(1,2:2:N);
%
% compute the DFT of x1: X1(k) = sum(x1(n)*WN^nk);
x2 = zeros(1,N); x3 = zeros(1,N);
% bitreverse input sequence
Indx = uint8(0:N-1);
N1 = log2(N);
for k = 0:uint8(N-1)
    Indx(k+1) = sum((2.^(uint8(N1)-uint8((1:N1))).*bitget(k,uint8(1:N1))));
end
x2 = x1(Indx+1);
%x2(1) = x1(1); x2(2) = x1(5); x2(3) = x1(3); x2(4) = x1(7);
%x2(5) = x1(2); x2(6) = x1(6); x2(7) = x1(4); x2(8) = x1(8);
% stage I: 2-point DFT
x3(1)= x2(1) + x2(2); x3(2)= x2(1) - x2(2);
x3(3) = x2(3) + x2(4); x3(4) = x2(3) - x2(4);
x3(5) = x2(5) + x2(6); x3(6) = x2(5) - x2(6);
x3(7) = x2(7) + x2(8); x3(8) = x2(7) - x2(8);
% stage II: 4-point DFT
W4 = exp(-2*pi*1i/4);
x2 = x3;
x2(3:4) = x2(3:4) .* (W4.^(0:1)); 
x2(7:8) = x2(7:8) .* (W4.^(0:1));
x3(1) = x2(1) + x2(3); x3(3) = x2(1) - x2(3);
x3(2) = x2(2) + x2(4); x3(4) = x2(2) - x2(4);
x3(5) = x2(5) + x2(7); x3(7) = x2(5) - x2(7);
x3(6) = x2(6) + x2(8); x3(8) = x2(6) - x2(8);
% third stage: 8-point DFT
W8 = exp(-2*pi*1i/8);
x2 = x3;
x2(5:8) = x2(5:8) .* (W8 .^ (0:3));
x3(1) = x2(1) + x2(5); x3(5) = x2(1) - x2(5);
x3(2) = x2(2) + x2(6); x3(6) = x2(2) - x2(6);
x3(3) = x2(3) + x2(7); x3(7) = x2(3) - x2(7);
x3(4) = x2(4) + x2(8); x3(8) = x2(4) - x2(8);
%multiply DFT by alpha(k) and W2N^(k/2) and take the real part
a(1) = 1/sqrt(N);
a(2:N) = sqrt(2/N);
W2N = exp(-1i*2*pi/(2*N));
y = real(a.*W2N .^((0:N-1)/2).*x3);






