function y = FastDFT(x,N)
% 1D FFT of an N-point sequence
%
% compute the DFT of x1: X1(k) = sum(x1(n)*WN^nk);
x2 = zeros(1,N); x3 = zeros(1,N);
% bitreverse input sequence
N1 = log2(N);
for k = 0:uint8(N-1)
    Indx(k+1) = sum((2.^(uint8(N1)-uint8((1:N1))).*bitget(k,uint8(1:N1))));
end
x2 = x(Indx+1);
%{
for s = 1:N1
    N2 = N /2^(N1-s);
    W = exp(-2*pi*1i/N2);
    n = 1;
    for m = 1:N/2
        y(n) = x2(n) + W*x2(n+N2-1);
        y(n+N2-1) = x2(n) -W*x2(n+N2-1);
        n = n + 2;
    end
    x2 = y;
%}
% stage I: 2-point DFT
x3(1)= x2(1) + x2(2); x3(2)= x2(1) - x2(2);
x3(3) = x2(3) + x2(4); x3(4) = x2(3) - x2(4);
x3(5) = x2(5) + x2(6); x3(6) = x2(5) - x2(6);
x3(7) = x2(7) + x2(8); x3(8) = x2(7) - x2(8);
% stage II: 4-point DFT
W4 = exp(-2*pi*1i/4);
x2 = x3;
x2(3:4) = x2(3:4) .* (W4.^(0:1)); 
x2(7:8) = x2(7:8) .* (W4.^(0:1));
x3(1) = x2(1) + x2(3); x3(3) = x2(1) - x2(3);
x3(2) = x2(2) + x2(4); x3(4) = x2(2) - x2(4);
x3(5) = x2(5) + x2(7); x3(7) = x2(5) - x2(7);
x3(6) = x2(6) + x2(8); x3(8) = x2(6) - x2(8);
% third stage: 8-point DFT
W8 = exp(-2*pi*1i/8);
x2 = x3;
x2(5:8) = x2(5:8) .* (W8 .^ (0:3));
x3(1) = x2(1) + x2(5); x3(5) = x2(1) - x2(5);
x3(2) = x2(2) + x2(6); x3(6) = x2(2) - x2(6);
x3(3) = x2(3) + x2(7); x3(7) = x2(3) - x2(7);
x3(4) = x2(4) + x2(8); x3(8) = x2(4) - x2(8);
%
y = x3;