function y = FastIDCT(x)
% y = FastIDCT(x)
% computes the idct of an N x N block
% using decimation in time IFFT
% N must be a power of 2, i.e., N = 2^m, where
% m is a positive integer

N = size(x,1);
x3 = zeros(N,N);
% multiply x by a(k)*W2N^(-k/2) & compute the idft of each row
W = exp(1i*pi/(2*N));
a(1) = 1/sqrt(N);
a(2:N) = sqrt(2/N);
for m = 1:N
    x(m,1:N) = a.*x(m,1:N).*W .^(0:N-1);
    x3(m,:) = ifft(x(m,:));
end
% multiply ifft(x) by N because ifft includes 1/N factor
x3 = x3 * N;
% reorder the output sequence to get the correct output sequence
y(:,1:2:N) = x3(:,1:N/2);
y(:,2:2:N)= x3(:,N:-1:N/2+1);
y = real(y);
%
% multiply y by a(k)*W2N^(-k/2) & compute the idft of each column
% which is the same thing as computing idft of y' along rows
y = y';
for m = 1:N
    x3(m,1:N) = a.*y(m,1:N).*W .^(0:N-1);
    x3(m,:) = ifft(x3(m,:));
end
x3 = x3 * N;
% reorder the output sequence to get the correct output sequence
y(:,1:2:N) = x3(:,1:N/2);
y(:,2:2:N)= x3(:,N:-1:N/2+1);
% even though y is supposed to be real, the imaginary parts
% of y will be zero and y will appear as complex.
% so, take the real part of y' to get the true IDCT
% remember that we transposed y in calculating the IDCT
% along columns. So, we have to transpose it to get the
% image in the normal order.
y = real(y');





