function y = FastIDCT(x)
% y = FastIDCT(x)
% computes the idct of an 8-point sequence
% using decimation in time IFFT

N = length(x);
W8 = exp(1i*pi/(2*N));
% multiply x by a(k)*W8^(-k/2)
a(1) = 1/sqrt(N);
a(2:N) = sqrt(2/N);
x = a.*x.*W8.^(0:N-1);
%
% compute IDFT of x
x2 = zeros(1,N); x3 = zeros(1,N);
% bitreverse input sequence
Indx = uint8(0:N-1);
N1 = log2(N);
for k = 0:uint8(N-1)
    Indx(k+1) = sum((2.^(uint8(N1)-uint8((1:N1))).*bitget(k,uint8(1:N1))));
end
x2 = x(Indx+1);
%x2(1) = x(1); x2(2) = x(5); x2(3) = x(3); x2(4) = x(7);
%x2(5) = x(2); x2(6) = x(6); x2(7) = x(4); x2(8) = x(8);
% stage I: 2-point DFT
x3(1)= x2(1) + x2(2); x3(2)= x2(1) - x2(2);
x3(3) = x2(3) + x2(4); x3(4) = x2(3) - x2(4);
x3(5) = x2(5) + x2(6); x3(6) = x2(5) - x2(6);
x3(7) = x2(7) + x2(8); x3(8) = x2(7) - x2(8);
% stage II: 4-point DFT
W4 = exp(2*pi*1i/4);
x2 = x3;
x2(3:4) = x2(3:4) .* (W4.^(0:1)); 
x2(7:8) = x2(7:8) .* (W4.^(0:1));
x3(1) = x2(1) + x2(3); x3(3) = x2(1) - x2(3);
x3(2) = x2(2) + x2(4); x3(4) = x2(2) - x2(4);
x3(5) = x2(5) + x2(7); x3(7) = x2(5) - x2(7);
x3(6) = x2(6) + x2(8); x3(8) = x2(6) - x2(8);
% third stage: 8-point DFT
W8 = exp(2*pi*1i/8);
x2 = x3;
x2(5:8) = x2(5:8) .* (W8 .^ (0:3));
x3(1) = x2(1) + x2(5); x3(5) = x2(1) - x2(5);
x3(2) = x2(2) + x2(6); x3(6) = x2(2) - x2(6);
x3(3) = x2(3) + x2(7); x3(7) = x2(3) - x2(7);
x3(4) = x2(4) + x2(8); x3(8) = x2(4) - x2(8);
%
y(1,1:2:N) = x3(1,1:N/2);
y(1,2:2:N)= x3(1,N:-1:N/2+1);
y = real(y);






