function [y1,y2,T] = KLTbasisImages(N,f)
% Karhunen-Loeve Transform of an image
%
[Height , Width] = size(f); 
%
b = zeros(N,N);
R = zeros(N,N);
Meanfc = zeros(1,N);
Meanfr = zeros(1,N);
% % compute the mean N-vector along column dimension 
for n = 1:N:Width
    for k = 1:N
        Meanfc(k) = Meanfc(k) + sum(f(:,n+k-1));
        Meanfr(k) = Meanfr(k) + sum(f(n+k-1,:),2);
    end
end
Meanfc = round(Meanfc/(Height*Width/N));        
% % compute the mean N-vector along row dimension
%{
for n = 1:N:Height
    for k = 1:N
        Meanfr(k) = Meanfr(k) + sum(f(n+k-1,:));
    end
end
%}
Meanfr = round(Meanfr/(Height*Width/N));
%
% calculate the N x N covariance matrix of N-vectors along column dimension
for m = 1:Height
    for n = 1:N:Width
        b = (f(m,n:n+N-1)-Meanfc)'*(f(m,n:n+N-1)-Meanfc);
        R = R + b;
    end
end
R = R/max(R(:));
[EVc,Evalc] = eig(R);% eigen matrix of R
% reorder the covariance matrix in descending order of eigen values
EVc = EVc(:,N:-1:1);
EVc = EVc(N:-1:1,:);
y2 = EVc;
% calculate the N x N covariance matrix of N-vectors along rows
R(:,:) = 0;
for n = 1:Width
    for m = 1:N:Height
        b = (f(m:m+N-1,n)-Meanfr')*(f(m:m+N-1,n)-Meanfr')';
        R = R + b;
    end
end
R = R/max(R(:));
[EVr,Evalr] = eig(R);% eign matrix of R
% reorder the covariance matrix in descending order of eigen values
EVr = EVr(:,N:-1:1);
EVr = EVr(N:-1:1,:);
y1 = EVr;
%
% Compute basis images and display
T = zeros(N*N,N*N);
for k = 0:N-1
    rInd = k*N + 1;
    for l = 0:N-1
        cInd = l*N +1;
        y = EVr(:,k+1) * EVc(:,l+1)';
        T(rInd:rInd+N-1,cInd:cInd+N-1) = y;
    end
end
