function y = UnitaryTransform(N,Type)
% Returns the N x N 1D unitary transform kernel
%
% Input:
% N = size of the basis image, assumed square
% For Haar transform, only N = 8 is allowed
% Type = unitary transform type
% Permissible transforms are:
% DCT, DST, Hartley, Haar, Hadamard, and Slant
%
y = zeros(N,N);
switch Type
    case 'DCT'
        for k = 0:N-1
            y(k+1,:) = cos((2*(0:N-1)+1)*pi*k/(2*N));
        end
        y(1,:) = y(1,:)/sqrt(N);
        y(2:N,:) = y(2:N,:) * sqrt(2/N);
    case 'DST'
        for k = 0:N-1
            y(k+1,:) = sin(((0:N-1)+1)*pi*(k+1)/(N+1));
        end
        y = y*sqrt(2/(N+1));
    case 'Hartley'
        for k = 0:N-1
            y(k+1,:) = cos((0:N-1)*2*pi*k/N) + sin((0:N-1)*2*pi*k/N);
        end
        y = y*sqrt(1/N);
    case 'Haar'
        sq2 = sqrt(2);
        y = [1 1 1 1 1 1 1 1; 1 1 1 1 -1 -1 -1 -1;...
            sq2 sq2 -sq2 -sq2 0 0 0 0; 0 0 0 0 sq2 sq2 -sq2 -sq2;...
            2 -2 0 0 0 0 0 0; 0 0 2 -2 0 0 0 0;...
            0 0 0 0 2 -2 0 0; 0 0 0 0 0 0 2 -2];
        y = y/sqrt(8);
    case 'Hadamard'
        y = [1 1; 1 -1];
        y = y/sqrt(2);
        n = 2;
        while n < N
            n = 2*n;
            y = [y y; y -y];
            y = y/sqrt(2);
        end
    case 'Slant'
        a8 = sqrt(16/21);
        b8 = sqrt(5/21);
        s4 = [1 1 1 1; 3/sqrt(5) 1/sqrt(5) -1/sqrt(5) -3/sqrt(5);...
            1 -1 -1 1; 1/sqrt(5) -3/sqrt(5) 3/sqrt(5) -1/sqrt(5)];
        s4 = s4/2;
        h1 = [1 0 0 0 1 0 0 0];
        h2 = [a8 b8 0 0 -a8 b8 0 0];
        h3 = [0 0 1 0 0 0 1 0];
        h4 = [0 0 0 1 0 0 0 1];
        h5 = [0 1 0 0 0 -1 0 0];
        h6 = [-b8 a8 0 0 b8 a8 0 0];
        h7 = [0 0 1 0 0 0 -1 0];
        h8 = [0 0 0 1 0 0 0 -1];
        y = [h1;h2;h3;h4;h5;h6;h7;h8]*[s4 zeros(4,4);...
            zeros(4,4) s4]/sqrt(2);
end