function BuildWaveletShape(WaveletName,Iter)
%   BuildWaveletShape(WaveletName,Iter)
%   Builds the wavelet and scaling function for the wavelet
%   WaveletName using Iter number of
%   iterations.
%   WaveletName can be one of 'db2','db4','db6', 'bior5.5', etc.


if ~exist('WaveletName','var')
    WaveletName = 'rbio5.5';
end
if ~exist('Iter','var')
    Iter = 1;
end
if nargin < 1
    error('at least one argument is required');
end
%
% Get the lowpass and highpass filters for the specified wavelet
[LP_D,HP_D] = wfilters(WaveletName,'d');
% upsample H and convolve it with L repeatedly Iter # times 
% to obtain the wavelet function
for i = 1:Iter
    Hu = dyadup(HP_D,2);
    h = conv(Hu,LP_D);
    HP_D = h;
end
HP_D = h;
% upsample L and convolve it with L repeatedly Iter # times 
% to obtain the scaling function
L1 = LP_D;
for i = 1:Iter
    Lu = dyadup(L1,2);
    L2 = conv(Lu,LP_D);
    L1 = L2;
end
LP_D = L2/sum(L2);% normalize the coefficients so the sum = 1 
%
% Repeat the procedure for the wavelet
[LP_R,HP_R] = wfilters(WaveletName,'r');
clear h Hu
for i = 1:Iter
    Hu = dyadup(HP_R,2);
    h = conv(Hu,LP_R);
    HP_R = h;
end
HP_R = h;
L1 = LP_R;
for i = 1:Iter
    Lu = dyadup(L1,2);
    L2 = conv(Lu,LP_R);
    L1 = L2;
end
LP_R = L2/sum(L2); 
%
N = length(LP_R);    % length of wavelet
%
figure,subplot(2,1,1),plot(0:N-1,HP_R,'k')
ylabel('Amplitude')
title([WaveletName ' :' 'Synthesis'  ' Wavelet function'])
subplot(2,1,2),plot(0:N-1,LP_R,'k')
ylabel('Amplitude')
title([WaveletName ' :' 'Synthesis' ' Scaling function'])
xlabel('Sample #')
%
figure,subplot(2,1,1),plot(0:N-1,HP_D,'k')
ylabel('Amplitude')
title([WaveletName ' :' 'Analysis'  ' Wavelet function'])
subplot(2,1,2),plot(0:N-1,LP_D,'k')
xlabel('Sample #')
ylabel('Amplitude')
title([WaveletName ' :' 'Analysis' ' Scaling function'])
