% OrthoDWT.m
% Properties of filter banks used in orthogonal DWT 

WaveletName = 'db8';
[LP_D,HP_D,LP_R,HP_R] = wfilters(WaveletName);
L = length(LP_D);
A = zeros(size(LP_D)); A(1) = 1;
% Filter length is even
% power complimentary condition
% time reversal and modulation on g0[n] & h0[n]
% time reversal and modulation on g1[n] & h1[n]
% h1[n] = (-1)^(n+1) x h0[L-1-n]
% g1[n] = (-1)^(n+1) x g0[L-1-n]
% sum(h0) = sum(g0) = sqrt(2)
sprintf('Filter length = %d',length(LP_D))
N = 256;
W = linspace(0,pi,N);
for n = 1:256
    s1 = 0; s2 = 0;
    s3 = 0; s4 = 0;
    for k = 1:L
        s1 = s1 + LP_R(k)*complex(cos((L/2-1-k)*W(n)),sin((L/2-1-k)*W(n)));
        s2 = s2 + HP_R(k)*complex(cos((L/2-1-k)*W(n)),sin((L/2-1-k)*W(n)));
        s3 = s3 + LP_D(k)*complex(cos((L/2-1-k)*W(n)),sin((L/2-1-k)*W(n)));
        s4 = s4 + HP_D(k)*complex(cos((L/2-1-k)*W(n)),sin((L/2-1-k)*W(n)));
    end
    G0(n) = s1;
    G1(n) = s2;
    H0(n) = s3;
    H1(n) = s4;
end
figure,plot(W,abs(G0),'k',W,abs(G1),'k',W,abs(G0).*abs(G0)+abs(G1).*abs(G1),'k')
title('Power Complimentary Condition:G0, G1')
figure,plot(W,abs(H0),'k',W,abs(H1),'k',W,abs(H0).*abs(H0)+abs(H1).*abs(H1),'k')
title('Power Complimentary Condition:H0, H1')
figure,plot(W,abs(H0),'k',W,abs(G1),'k',W,abs(H0).*abs(H0)+abs(G1).*abs(G1),'k')
title('Power Complimentary Condition:H0, G1')
figure,plot(W,abs(G0),'k',W,abs(H1),'k',W,abs(G0).*abs(G0)+abs(H1).*abs(H1),'k')
title('Power Complimentary Condition:G0, H1')
n = 0:L-1;
sprintf('time reversal & modulation: %2.3f',(-1).^n .*LP_R(L-n))
sprintf('sum(h0) = %2.3f or sum(g0) = %2.3f',sum(LP_D),sum(LP_R))
figure,subplot(4,1,1),plot(LP_D,'k','LineWidth', 2); title('DB8 Decomposition LPF')
subplot(4,1,2),plot(HP_D,'k','LineWidth', 2); title('DB8 Decomposition HPF')
subplot(4,1,3),plot(LP_R,'k','LineWidth', 2); title('DB8 Reconstruction LPF')
subplot(4,1,4),plot(HP_R,'k','LineWidth', 2); title('DB8 Reconstruction HPF')