% ReversibleDWT.m
% Reversible Discrete Wavelet Transform
% This is a biorthogonal DWT and is called
% LeGall's 5/3 or CDF 5/3 filters and is used in
% JPEG 2000 standard for lossless compression.

% The analysis lowpass filter h0 has 5 taps,
% analysis highpass filter has 3 taps,
% sysnthesis lowpass filter has 3 taps and
% synthesis highpass filter has 5 taps.
h0 = [-1/8 2/8 6/8 2/8 -1/8];
h1 = [-1/2 1 -1/2];
g0 = [1/2 1 1/2];
g1 = [-1/8 -2/8 6/8 -2/8 -1/8];
%
A1 = zeros(1,5);
A1(1) = 1;
A2 = zeros(1,3);
A2(1) = 1;
[H0,W] = freqz(h0,A1,256);
[H1,W] = freqz(h1,A2,256);
[G0,W] = freqz(g0,A2,256);
[G1,W] = freqz(g1,A1,256);
%
figure,plot(W,abs(H0),'k','LineWidth',2)
hold on
plot(W,abs(H1),'--k','LineWidth',2)
title('LeGall 5-3 Analysis Filterbank')
legend('Lowpass', 'Highpass',0)
xlabel('Normalized frequency')
ylabel('Amplitude')
hold off
figure,plot(W,angle(H0),'k','LineWidth',2)
hold on
plot(W,angle(H1),'--k','LineWidth',2)
title('LeGall 5-3 Analysis Filterbank')
legend('Lowpass', 'Highpass',0)
xlabel('Normalized frequency')
ylabel('Phase')
hold off
%
figure,plot(W,abs(G0),'k','LineWidth',2)
hold on
plot(W,abs(G1),'--k','LineWidth',2)
title('Synthesis Filterbank')
legend('Lowpass', 'Highpass',0)
xlabel('Normalized frequency')
ylabel('Amplitude')
hold off
figure,plot(W,angle(G0),'k','LineWidth',2)
hold on
plot(W,angle(G1),'--k','LineWidth',2)
title('LeGall 5-3 Synthesis Filterbank')
legend('Lowpass', 'Highpass',0)
xlabel('Normalized frequency')
ylabel('Phase')
hold off