% Example5_5.m
% Example5_5 encodes a symbol sequence "m" using arithmetic coding. 
% It generates a tag in a truncated bianry fraction
% to a minimum number of bits "TB" of accuracy and
% decodes the input sequence.
% "p" is the probability vector of the source
% alphabet; "m" is the vector (of integers) whose
% elements are the symbols in the sequence. The values
% in "m" must be integers between 1 & length(p).
% "Tag" is the truncated value of the tag for the
% encoded sequence.

p = [0.5 0.3 0.1 0.1];% probability vector of input alphabet
if any(find(p<0))
    error('Negative value for probability\n')
end
if abs(sum(p)-1) >1.0e-10
    error('Not a valid probability set\n')
end
m = [1 3 2 4 1];% symbol sequence to be coded
%m = [1 3 2 1 4 2 1 1];
%TB is the significant number of bits of the Tag
TB = ceil(-log2(prod(p(m))))+1;
sprintf('Precision of Tag = %g bits\n',TB)
sprintf('Entropy = %4.3f;\tAvg. Code Length = %4.3f\n',...
    sum(-p.*log2(p)),TB/length(m)) 
sprintf('Efficiency = %4.2f\n',(100*sum(-p.*log2(p))/TB*length(m)))
% compute the cumulative probability function (CDF) of p
CDF = zeros(1,length(p));
for i = 1:length(p)
    CDF(i) = sum(p(1:i));
end
%
% Compute the Tag corresponding to the symbol sequence
% by calling the function "ArithmeticEncode"
Tag = ArithmeticEncode(CDF,m);
% generate truncated code for the tag
AC = uint16(0);% contains the arithmetic code
k = 1;
T = Tag;
while k <=TB
    T = 2*T;
    if T>= 1
        AC = bitshift(AC,1);
        AC = AC + uint16(1);
        T = T -1;
    else
        AC = bitshift(AC,1);
    end
    k = k + 1;
end
% Now decode the symbol sequence
T1 = 0.0;
AC1 = AC;
TB1 = TB + 1;
for k = 1:TB
    T1 = double(bitand(AC1,1))*(2^(k-TB1)) + T1;
    AC1 = bitshift(AC1,-1);
end
sprintf('Tag = %10.8f\t\tTruncated Tag = %10.8f\t\tArithmetic code = %g\n',...
    Tag,T1,AC)
%
D = ArithmeticDecode(CDF,T1,length(m));
sprintf('\nEncoded Symbol Sequence :\n')
sprintf('%g\t',m)
sprintf('\nDecoded Symbol Sequence :\n')
sprintf('%g\t',D)