function ADPCM_Classify(A,B)
% ADPCM_Classify(A,B)
% Adaptive 1D DPCM using pixel classifier
% Classification is based on the variance of a
% neighborhood of pixels centered on the current pixel.
% Uses 3 uniform quantizers with different bits 
% of quantization.
% Input:
% A = intensity image
% B = 3-vector whose elements are the # bits of each quantizer 


[Height,Width,Depth] = size(A);
if Depth > 1
    A1 = double(A(:,:,1));
else
    A1 = double(A);
end
[alfa,E] = LinearPredict_1D(A);
%
% Design 3 uniform quantizers
dMin(1) = 0;
dMax(1) = 0.3*std2(E);
dMin(2) = dMax(1);
dMax(2) = 1.05*std2(E);
dMin(3) = dMax(2);
dMax(3) = 5*std2(E);
%
L(1) = 2^B(1); L(2) = 2^B(2); L(3) = 2^B(3);
q = zeros(3);
q2 = zeros(3);
rL1(1:L(1)) = 0; % reconstruction levels
rL2(1:L(2)) = 0;
rL3(1:L(3)) = 0;
%
q(1) = 2*dMax(1)/L(1);
q2(1) = q(1)/2;
dR1 = linspace(dMin(1),dMax(1),L(1)+1); % decision intervals
%
for k = 1:L(1)
    rL1(k) = dR1(k)+q2(1);
end
%
q(2) = 2*dMax(2)/L(2);
q2(2) = q(2)/2;
dR2 = linspace(dMin(2),dMax(2),L(2)+1);
%
for k = 1:L(2)
    rL2(k) = dR2(k)+q2(2);
end
%
q(3) = 2*dMax(3)/L(3);
q2(3) = q(3)/2;
dR3 = linspace(dMin(3),dMax(3),L(3)+1);
%
for k = 1:L(3)
    rL3(k) = dR3(k)+q2(3);
end
%
Mu = mean2(A1);% mean value of the image
% Implement the ADPCM
pe = zeros(Height,Width);% array to store differential image
y = zeros(Height,Width);% array to store reconstructed image
peq = zeros(Height,Width);% array to store quantized differential image
A1 = A1 - Mu;% mean removed input image
SigVar = (std2(A1))^2; % input signal variance
padRows = 3; padCols = 3;
% extend the input image with padRows and padCols all around
A1 = padarray(A1,[padRows padCols],'symmetric','both');
Pcount = zeros(1,3); % percent of each class
for r = padRows:Height+padRows-1
    y(r-padRows+1,1) = A1(r-padRows+1,1) + Mu; % first pixel unquantized
    x1 = A1(r-padRows+1,1); % previous pixel
    for c = padCols:Width+padCols-1
        xhat = alfa*x1; % predicted pixel
        pe(r,c) = A1(r,c) - xhat; % differential pixel
        % local variance for classification
        BlkVar = std2(A1(r-padRows+1:r+padRows-1,c-padCols+1:c+padCols-1))^2;
        s = 1;
        if pe(r,c) < 0
            s = -1;
        end
        e = abs(pe(r,c));
        if BlkVar < 9 % Class I
            Pcount(1) = Pcount(1) + 1;
            for k = 1:L(1)
                if e>dR1(k) && e<=dR1(k+1)
                    peq(r,c) = s*rL1(k);
                elseif e<= dR1(1)
                        peq(r,c) = s*rL1(1);
                elseif e > dR1(L(1)+1)
                    peq(r,c) = s*rL1(L(1));
                end
            end
        elseif BlkVar >= 9 && BlkVar < 1225 % Class II
            Pcount(2) = Pcount(2) + 1;
            for k = 1:L(2)
                if e>dR2(k) && e<=dR2(k+1)
                    peq(r,c) = s*rL2(k);
                elseif e<= dR2(1)
                        peq(r,c) = s*rL2(1);
                elseif e > dR2(L(2)+1)
                    peq(r,c) = s*rL2(L(2));
                end
            end
        else % Class III
            Pcount(3) = Pcount(3) + 1;
            for k = 1:L(3)
                if e>dR3(k) && e<=dR3(k+1)
                    peq(r,c) = s*rL3(k);
                elseif e<= dR3(1)
                        peq(r,c) = s*rL3(1);
                elseif e > dR3(L(3)+1)
                    peq(r,c) =s* rL3(L(3));
                end
            end
        end
        x1 = peq(r,c) + xhat; % previously reconstructed pixel
        y(r-padRows+1,c-padCols+1) = x1 + Mu;% mean added reconstructed pixel
    end
end
Pcount = Pcount/sum(Pcount);
% display the reconstructed image & calculate SNR
figure,imshow(y,[]), title('Reconstructed image')
SNR = 10*log10(SigVar/((std2(A1(padRows:Height+padRows-1,padCols:Width+padCols-1) - y))^2));
sprintf('SNR = %4.2f\n',SNR)
sprintf('Class 1 = %5.2f%%\tClass 2 = %5.2f%%\tClass 3 = %5.2f%%\n',Pcount*100)
%
% display histogram of quantized & unquantized differential images
[H,B] = hist(pe(:),512);
H = H/sum(H);
[H1,B1] = hist(peq(:),512);
H1 = H1/sum(H1);
figure,subplot(2,1,1),plot(B,H,'k','LineWidth',2)
title('Histogram of unquantized differential image')
ylabel('relative count')
subplot(2,1,2),plot(B1,H1,'k','LineWidth',2)
title('Histogram of quantized differential image')
xlabel('pixel value'),ylabel('relative count')