function [alfa,E,pe,peq,y] = DPCM_1D(f,Qtype,B)
% [alfa,E,y] = DPCM_1D(f,Qtype,B)
% Designs and implements 1D DPCM
% Input:
%   f = intensity image to be encoded
%   Qtype = quantizer type: "uniform" or "nonuniform"
%   B = # bits of quantization
%
% Output:
%   alfa = predictor coefficient
%   E = unquantized prediction error image
%   pe = unquantized differential image
%   peq = quantized differential image
%   y = DPCM reconstructed image
%
% This function implements only the encoder.

L = 2^B;
[Height,Width] = size(f);
[alfa,E] = LinearPredict_1D(f);% design the linear predictor
switch Qtype
    case 'uniform'
        dMin = mean2(E) - 5*std2(E);
        dMax = mean2(E) + 5*std2(E);
        q = 2*dMax/L;
        q2 = q/2;
        dR = linspace(dMin,dMax,L+1);
        rL = zeros(L,1);
        for k = 1:L
            rL(k) = dR(k)+q2;
        end
    case 'nonuniform'
        [DR,C] = dpcmQuantizer(E,B);% design a B-bit non-uniform quantizer
end
%
Mu = mean2(f);% mean value of the image
% Implement the DPCM
pe = zeros(Height,Width);% array to store differential image
y = zeros(Height,Width);% array to store reconstructed image
peq = zeros(Height,Width);% array to store quantized differential image
f = f - Mu;% mean removed input image
for r = 1:Height
    y(r,1) = f(r,1) + Mu; % first pixel unquantized
    x1 = f(r,1); % previous pixel
    for c = 2:Width
        xhat = alfa*x1; % predicted pixel
        pe(r,c) = f(r,c) - xhat; % differential pixel
        switch Qtype
            case 'uniform'
                for k = 1:L
                    if pe(r,c)>dR(k) && pe(r,c)<=dR(k+1)
                        peq(r,c) = rL(k);
                    elseif pe(r,c)<= dR(1)
                            peq(r,c) = rL(1);
                    elseif pe(r,c) > dR(L+1)
                        peq(r,c) = rL(L);
                    end
                end
            case 'nonuniform'
                for k = 1:L
                    if (pe(r,c)>DR(k) && pe(r,c)<=DR(k+1))
                            peq(r,c) = C(k);
                    end
                end
        end
        x1 = peq(r,c) + xhat; % previously reconstructed pixel
        y(r,c) = x1 + Mu;% mean added reconstructed pixel
    end
end