% Example6_1.m
% Designs a 1st-order DPCM (encoder only)using both
% uniform and non-uniform quantizers with user specified
% number of bits of quantization. It also calculates
% the prediction gain and bit rate reduction. It further
% displays the histograms of the quantized and unquantized
% prediction error images as well as the reconstructed
% image.Finally, the SNR(dB) due to DPCM encoding is
% calculated.


clear, %close all
A = imread('cameraman.tif');
[Height,Width,Depth] = size(A);
if Depth > 1
    A1 = double(A(:,:,1));
else
    A1 = double(A);
end
Qtype = 'uniform'; % 'uniform' and 'nonuniform'
B = 5; % # bits of quantization
% Design & implement the 1D DPCM
[alfa,E,pe,peq,A2] = DPCM_1D(A1,Qtype,B);
%
% display histogram of quantized & unquantized differential images
[H,B] = hist(pe(:),512);
H = H/sum(H);
[H1,B1] = hist(peq(:),512);
H1 = H1/sum(H1);
figure,subplot(2,1,1),plot(B,H,'k','LineWidth',2)
title('Histogram of unquantized differential image')
ylabel('relative count')
subplot(2,1,2),plot(B1,H1,'k','LineWidth',2)
title('Histogram of quantized differential image')
xlabel('pixel value'),ylabel('relative count')
% display the reconstructed image
figure,imshow(A2,[]), title('Reconstructed image')
% Compute Gp and rate reduction
Evar = sum(sum(E.* E))/(Height*Width);
SigVar = (std2(A1))^2;
sprintf('Pred. Coeff. = %5.4f\n',alfa)
Gp = 10*log10(SigVar/Evar);
RateRed = 0.5*log2(SigVar/Evar);
sprintf('Gp = %4.2f dB\tRateRed = %4.2f bits\n',Gp,RateRed)
SNR = 10*log10(SigVar/((std2(A1 - A2))^2));
sprintf('SNR = %4.2f\n',SNR)
    