function [alfa,pe] = LinearPredict_2D(I)
% [alfa,pe] = LinearPredict_2D(I)
% I = input intensity image
% alfa = predictor coefficient vector
% pe = prediction error image of the same size as I
% Uses 2D linear prediction to
% create the differential image
% xhat(m,n) = alfa1(1)*x(m,n-1)+alfa(2)*x(m-1,n)+...
%    alfa(3)*x(m-1,n-1)+alfa(4)*x(m-1,n+1)


[Height,Width,Depth] = size(I);
if Depth > 1
    I1 = double(I(:,:,1));
else
    I1 = double(I);
end
%
Mu = mean2(I1);% mean value of the image
I1 = I1 - Mu;% mean removed input image
r00 = mean2(I1 .* I1);
r01 = sum(sum(I1(:,2:Width).* I1(:,1:Width-1)))/(Height*(Width-1));
r10 = sum(sum(I1(2:Height,:).* I1(1:Height-1,:)))/((Height-1)*Width);
r11 = sum(sum(I1(2:Height,2:Width).* I1(1:Height-1,1:Width-1)))/((Height-1)*(Width-1));
r12 = sum(sum(I1(2:Height,3:Width).* I1(1:Height-1,1:Width-2)))/((Height-1)*(Width-2));
r02 = sum(sum(I1(1:Height,3:Width).* I1(1:Height,1:Width-2)))/(Height*(Width-2));
alfa = inv([r00 r11 r10 r12; r11 r00 r01 r01;...
    r10 r01 r00 r02; r12 r01 r02 r00])*[r01 r10 r11 r11]';
% Implement the Linear Predictor
pe = zeros(Height,Width);% array to store prediction errors
%I1 = padarray(I1,[1 1],'symmetric','pre');
I1 = padarray(I1,[1 2],'symmetric','pre');
for r = 2:Height+1
    for c = 2:Width+1
        xhat = alfa(1)*I1(r,c-1)+alfa(2)*I1(r-1,c)+ ...
            alfa(3)*I1(r-1,c-1) + alfa(4)*I1(r-1,c+1);
        pe(r-1,c-1) = I1(r,c) - xhat;% differential pixel
    end
end