function [DR,C] = dpcmQuantizer(x,b)
%   [DR,C] = dpcmQuantizer(x,b)
% Design of a non-uniform quantizer for the DPCM
% x = input differential image
% b = number of bits of quantization
% DR = input decision boundaries
% C = output reconstruction levels


[Height,Width] = size(x);
L = 2^b;
C = linspace(min(x(:)),max(x(:)),L);
%C = linspace(min(x(:))+2,max(x(:))-2,L);
PartitionRegion = zeros(L,Height,Width);
RegionCount = zeros(L,1);
RegionSum = zeros(L,1);
Distortion = zeros(L,1);
TotalDistortion = 1.0e+5; % total minimum distortion
err = 0.1e-01; % accuracy to terminate iteration
TD = (TotalDistortion-sum(Distortion))/TotalDistortion;
% start iteration
It = 1; % iteration number
while TD >= err
    for r = 1:Height
        for c = 1:Width
            d1 = abs(x(r,c)-C(1));
            J = 1;
            for k = 2:L
                d2 = abs(x(r,c)-C(k));
                if d2<d1
                    d1 = d2;
                    J = k;
                end
            end
            RegionCount(J) = RegionCount(J) + 1;
            RegionSum(J) = RegionSum(J) + x(r,c);
            PartitionRegion(J,r,c) = x(r,c);
            Distortion(J) = Distortion(J)+(x(r,c)-C(J))*(x(r,c)-C(J));
        end
    end
    Index = logical(RegionCount == 0);% check for empty regions
    RegionCount(Index) = 1;% set empty regions to count 1
    Distortion = Distortion ./RegionCount;% average minimum distortions
    %TD = abs(TotalDistortion-sum(Distortion));
    % alternate convergence measure
    TD = (TotalDistortion-sum(Distortion))/TotalDistortion;
    C = RegionSum ./RegionCount; % refined output levels
    TotalDistortion = sum(Distortion);% total minimum distortion 
    It = It + 1;
end
clear Index
DR = zeros(L+1,1);
DR(L+1) = 255;
for k = 2:L
    DR(k) = (C(k-1)+C(k))/2;
end