function Qbits = AllocateBits(A,R,N)
% Qbits = AllocateBits(A,R)
% Computes the quantizer bits optimally using the rule:
% R(k) = R +0.5*log2(var of kth coeff./geometric variance)
% where R in bpp is the bit budget.

% N is the size of the unitary transform matrix
[Height,Width] = size(A);
CoefSigma = zeros(N,N); % Std. dev. of the transform coefficients
Qbits = zeros(N,N); % Array containing the quantizer bits
fun = @dct2;
T = blkproc(A,[N N],fun);   % do N x N 2D DCT
%   compute the DCT coefficient variance
for k = 1:N
    for l = 1:N
        CoefSigma(k,l) = std2(T(k:N:Height,l:N:Width));
    end
end
%
P1 = CoefSigma .^ (1/(N^2));
GeoMeanOfSigma = prod(P1(:)); % geometric mean of the coef. variances
for k = 1:N
    for l = 1:N
        b = R + log2(CoefSigma(k,l)/GeoMeanOfSigma);
        if b <= 0
            Qbits(k,l) = 0;
        else
            Qbits(k,l) = round(b);
        end        
    end
end