function Qbits = AssignIntgrBits(x,R,N)
%   Assigns quantizer bits optimally using
%   recursive integer bit allocation rule.


M = N*N; % total number of quantizers
Rtotal = M*R;   % total bits for the N x N block
[Rows,Cols] = size(x); % Rows & Columns of input image
fun = @dct2;
T = blkproc(x,[N N],fun);   % do N x N 2D DCT
%   compute the DCT coefficient variances
VarOfCoef = zeros(N,N);
for k = 1:N
    for l = 1:N
        V = std2(T(k:N:Rows,l:N:Cols));
        VarOfCoef(k,l) = V * V;
    end
end
%   assign integer bits recursively
Qbits = zeros(N,N);
Var = VarOfCoef;
while Rtotal > 0
    Max = -9999;
    for k = 1:N
        for l = 1:N
            if Max < Var(k,l)
                Max = Var(k,l);
                Indx = [k l];
            end
        end
    end
    Qbits(Indx(1),Indx(2)) = Qbits(Indx(1),Indx(2)) + 1;
    Var(Indx(1),Indx(2)) = Var(Indx(1),Indx(2))/2;
    Rtotal = Rtotal - 1;
end
%
sprintf('Optimal quantizer bits\n')
disp(Qbits)
sprintf('Desired avg. rate = %3.2f bpp\tActual avg. rate = %3.2f bpp\n',R,sum(Qbits(:))/(N*N))
