% BlockingArt.m

clear
%A = imread('birds.ras');  % read an image
A = imread('masuda2.ras');
[Height,Width,Depth] = size(A);
%
N = 8; % Transform matrix size
if mod(Height,N)~= 0
    Height = floor(Height/N)*N;
end
if mod(Width,N)~= 0
    Width = floor(Width/N)*N;
end
A1 = A(1:Height,1:Width,:);
clear A
A = A1;
if Depth == 1
    A1 = double(A);
else
    A = double(rgb2ycbcr(A));
    A1 = A(:,:,1);
end
Qtype = 'jpg'; % "jpg", "mpg", or "opti"
Y = blkproc(double(A1),[N N],@dct2); % N x N 2D DCT of input image
% call function AllocateBits to determine quantizer optimal bits
switch Qtype
    case 'opti'
        R = 1.0; % Bit budget in bpp
        Qbits = AllocateBits(A1,R,N);
        [x,y] = find(Qbits == 0); % positions of zero coefficients
        % Determine the quantization steps of uniform quantizers
        dcMax = max(max(Y(1:N:Height,1:N:Width)));
        Qsteps = zeros(N,N);
        for k = 1:N
            for l = 1:N
                acMax = max(max(Y(k:N:Height,l:N:Width)));
                Qsteps(k,l) = round(acMax /(2*(2 ^ Qbits(k,l))));
            end
        end
        Qsteps(1,1) = round(dcMax/(2*(2 ^ Qbits(1,1))));
    case 'jpg'
        Qsteps = [16 11 10 16 24 40 51 61;...
            12 12 14 19 26 58 60 55;...
            14 13 16 24 40 57 69 56;...
            14 17 22 29 51 87 80 62;...
            18 22 37 56 68 109 103 77;...
            24 35 55 64 81 104 113 92;...
            49 64 78 87 103 121 120 101;...
            72 92 95 98 112 100 103 99];
    case 'mpg'
        Qsteps = [8 16 19 22 26 27 29 34;...
            16 16 22 24 27 29 34 37;...
            19 22 26 27 29 34 34 38;...
            22 22 26 27 29 34 37 40;...
            22 26 27 29 32 35 40 48;...
            26 27 29 32 35 40 48 58;...
            26 27 29 34 38 46 56 69;...
            27 29 35 38 46 56 69 83];
end
%
Yq = zeros(Height,Width); % array to store quantized DCT coefficients
temp = zeros(N,N);
eVar = zeros(1,19);
% quantize & dequantize the DCT coefficients
k = 1;
for Qscale = 5.0:0.25:5.0
    for m = 1:N:Height
        for n = 1:N:Width
            temp = floor(Y(m:m+N-1,n:n+N-1)./(Qscale*Qsteps)...
                + 0.5).*(Qscale*Qsteps);
            if strcmpi(Qtype,'opti')
                for i = 1:length(x)
                    temp(x(i),y(i)) = 0;
                end
            end
            Yq(m:m+N-1,n:n+N-1) = temp;
        end
    end
    %
    xq = blkproc(Yq,[N N],@idct2); % 2D IDCT
    %
    dr = A1(1:N:Height,:) - xq(1:N:Height,:);
    dr = dr(:) .* dr(:) / (Height*Width/N);
    dc = A1(:,1:N:Width) - xq(:,1:N:Width);
    dc = dc(:) .* dc(:) / (Height*Width/N);
    eVar(k) = sum((dr + dc))/2;
    k = k + 1;
end
figure,plot(1.0:0.5:10.0,sqrt(eVar),'k','LineWidth', 2)
figure,imshow(xq,[])