% TCcompress.m

clear
A = imread('cameraman.tif');  % read an image
N = 8; % Transform matrix size
Yq = zeros(256,256); % array to store quantized DCT coefficients
temp = zeros(N,N);
Qtype = 'jpg'; % Qtype = 'MPG', 'JPG', or 'OPTI'
%
Qbits = [6 4 3 2 2 1 1 1;...
    3 3 2 2 2 2 1 1;...
    3 2 2 2 1 1 1 1;...
    2 1 1 1 1 1 1 0;...
    2 1 1 1 1 0 0 0;...
    1 1 0 0 0 0 0 0;...
    1 0 0 0 0 0 0 0;...
    1 0 0 0 0 0 0 0];
[x,y] = find(Qbits == 0);
optiQsteps = [12 18 29 35 30 46 32 22;...
    32 22 23 22 28 24 53 38;...
    20 23 23 17 39 29 27 26;...
    32 27 32 26 35 25 21 31;...
    34 19 27 25 27 47 22 34;...
    19 14 37 23 34 47 35 19;...
    38 30 33 27 18 31 18 18;...
    18 36 26 25 14 19 16 23];
    
jpgQsteps = [16 11 10 16 24 40 51 61;...
    12 12 14 19 26 58 60 55;...
    14 13 16 24 40 57 69 56;...
    14 17 22 29 51 87 80 62;...
    18 22 37 56 68 109 103 77;...
    24 35 55 64 81 104 113 92;...
    49 64 78 87 103 121 120 101;...
    72 92 95 98 112 100 103 99];
%
mpg1Qsteps = [8 16 19 22 26 27 29 34;...
    16 16 22 24 27 29 34 37;...
    19 22 26 27 29 34 34 38;...
    22 22 26 27 29 34 37 40;...
    22 26 27 29 32 35 40 48;...
    26 27 29 32 35 40 48 58;...
    26 27 29 34 38 46 56 69;...
    27 29 35 38 46 56 69 83];
%
switch Qtype
    case {'JPG','jpg'}
        Qsteps = jpgQsteps;
    case {'MPG','mpg'}
        Qsteps = mpg1Qsteps;
    case {'OPTI','opti'}
        Qsteps = optiQsteps;
end
Scale = 1.0;
% Compute 2D DCT of the input image
Y = blkproc(double(A),[N N],@dct2);
%
% quantize & dequantize the DCT coefficients
TotalBits = int32(0);
for m = 1:N:256
    for n = 1:N:256
        s = sign(Y(m:m+N-1,n:n+N-1)); % sign of coefficients
        temp = floor(abs(Y(m:m+N-1,n:n+N-1))./(Qsteps*Scale) + 0.5);
        if strcmpi(Qtype,'opti')
            temp(x,y) = 0;
        end
        % calculate bits for differential DC coefficient
        if n == 1
            DC = temp(1,1);
            DCbits = round(log2(DC));
        else
            DC = temp(1,1) - DC;
            DCbits = dcBits(DC);
        end
        % calculate bits for AC coefficients
        ACbits = acBlockBits(temp);
        TotalBits = TotalBits + DCbits + ACbits;
        Yq(m:m+N-1,n:n+N-1) = s.*temp .*(Qsteps*Scale);
        DC = temp(1,1);
    end
end
AvgBits = double(TotalBits)/(256*256);
sprintf('Average bit rate = %3.2f\n', AvgBits)
%
xq = blkproc(Yq,[8 8],@idct2); 
figure,imshow(xq,[]), title([Qtype ' Quantized image'])
SNR = 20*log10(std2(A)/std2(double(A) - xq));
sprintf('SNR = %4.2f\n', SNR)