% TCexample.m

clear
A = imread('cameraman.tif');  % read an image
R = 1.0; % Bit budget in bpp
N = 8; % Transform matrix size
% call function AllocateBits to determine quantizer optimal bits
Qbits = AllocateBits(A,R,N);
[x,y] = find(Qbits == 0); % positions of zero coefficients
Y = blkproc(double(A),[N N],@dct2); % N x N 2D DCT of input image
%
% Determine the quantization steps of uniform quantizers
dcMax = max(max(Y(1:N:256,1:N:256)));
Qsteps = zeros(N,N);
for k = 1:N
    for l = 1:N
        acMax = max(max(Y(k:N:256,l:N:256)));
        Qsteps(k,l) = round(acMax /(2*(2 ^ Qbits(k,l))));
    end
end
Qsteps(1,1) = round(dcMax/(2*(2 ^ Qbits(1,1))));
jpgQsteps = [16 11 10 16 24 40 51 61;...
    12 12 14 19 26 58 60 55;...
    14 13 16 24 40 57 69 56;...
    14 17 22 29 51 87 80 62;...
    18 22 37 56 68 109 103 77;...
    24 35 55 64 81 104 113 92;...
    49 64 78 87 103 121 120 101;...
    72 92 95 98 112 100 103 99];
%
mpg1Qsteps = [8 16 19 22 26 27 29 34;...
    16 16 22 24 27 29 34 37;...
    19 22 26 27 29 34 34 38;...
    22 22 26 27 29 34 37 40;...
    22 26 27 29 32 35 40 48;...
    26 27 29 32 35 40 48 58;...
    26 27 29 34 38 46 56 69;...
    27 29 35 38 46 56 69 83];
%
Yq = zeros(256,256); % array to store quantized DCT coefficients
YqJPG = zeros(256,256);
YqMPG = zeros(256,256);
temp = zeros(N,N);
% quantize & dequantize the DCT coefficients
for m = 1:N:256
    for n = 1:N:256
        s = sign(Y(m:m+N-1,n:n+N-1)); % sign of coefficients
        temp = floor(abs(Y(m:m+N-1,n:n+N-1))./Qsteps + 0.5).*Qsteps;
        temp(x,y) = 0;
        Yq(m:m+N-1,n:n+N-1) = s.*temp;
        temp = floor(abs(Y(m:m+N-1,n:n+N-1))./jpgQsteps + 0.5).*jpgQsteps;
        YqJPG(m:m+N-1,n:n+N-1) = s.*temp;
        temp = floor(abs(Y(m:m+N-1,n:n+N-1))./mpg1Qsteps + 0.5).*mpg1Qsteps;
        YqMPG(m:m+N-1,n:n+N-1) = s.*temp;
    end
end
%
xq = blkproc(Yq,[8 8],@idct2); % 2D IDCT
xqJPG = blkproc(YqJPG,[8 8],@idct2); 
xqMPG = blkproc(YqMPG,[8 8],@idct2); 
figure,imshow(xq,[]), title(['Quantized image @ ' num2str(R) ' bpp'])
figure,imshow(xqJPG,[]), title('JPEG Quantized image')
figure,imshow(xqMPG,[]), title('MPEG1 Quantized image')
SNR = 20*log10(std2(A)/std2(double(A) - xq));
SNRjpg = 20*log10(std2(A)/std2(double(A) - xqJPG));
SNRmpg = 20*log10(std2(A)/std2(double(A) - xqMPG));
sprintf('SNR @ %3.2f bpp = %4.2f\n', R, SNR)
sprintf('SNR for JPEG = %4.2f\n', SNRjpg)
sprintf('SNR for JPEG = %4.2f\n', SNRmpg)