function Bits = acBlockBits(x)
% y = acBlockBits(x)
% compress and decompress an N x N DCT block x
% using MPEG1 quantization matrix.
% AC coefficients are zigzag scanned and
% the RL/Amplitude pairs are found. Then,
% using the MPEG1 RL/Amp table, number of bits to
% compress the block is determined.
% Input:
%   x = N x N DCT block
% Outputs:
%   y = decompressed block
%   Bits = total bits to encode the block
%
% MPEG lengths of variable-length codes for run/level values
% run-length 0 has 40 levels
% run-length 1 has 18 levels
% run-length 2 has 5 levels
% run-length 3 has 4 levels
% run-lengths 4 through 6 have 3 levels
% run-lengths 7 through 16 have 2 levels
% run-lengths 18 through 31 have 1 level
% run-lengths not in the above list will use escape code
% plus 6 bits for run-length plus 8 bits for level
%
RLC = cell(1,34);
RLC{1}=int16([3 5 6 8 9 9 11 13 13 13 13 14 14 14 14 15 15 15 15 15 ...
    15 15 15 15 15 15 15 15 15 15 15 16 16 16 16 16 16 16 16 16]);
RLC{2} = int16([4 7 9 11 13 14 14 16 16 16 16 16 16 16 17 17 17 17]);
RLC{3} = int16([5 8 11 13 14]);
RLC{4} = int16([6 9 13 14]);
RLC{5} = int16([6 11 13]);
RLC{6} = int16([7 11 14]);
RLC{7} = int16([7 13 17]);
RLC{8} = int16([7 13]);
RLC{9} = int16([8 13]);
RLC{10} = int16([8 14]);
RLC{11} = int16([9 14]);
RLC{12} = int16([9 17]);
RLC{13} = int16([9 17]);
RLC{14} = int16([9 17]);
RLC{15} = int16([11 17]);
RLC{16} = int16([11 17]);
RLC{17} = int16([11 17]);
RLC{18} = int16(13);
RLC{19} = int16(13);
RLC{20} = int16(13);
RLC{21} = int16(13);
RLC{22} = int16(13);
RLC{23} = int16(14);
RLC{24} = int16(14);
RLC{25} = int16(14);
RLC{26} = int16(14);
RLC{27} = int16(14);
RLC{28} = int16(17);
RLC{29} = int16(17);
RLC{30} = int16(17);
RLC{31} = int16(17);
RLC{32} = int16(17);
RLC{33} = int16(2); % EOB
RLC{34} = int16(6); % Escape
%
% zigzag scan the 8 x 8 DCT blocks
x1 = ZigZag(x);
%
k = 2; Count = 0; Bits = int32(0);
while k <= 64
    if x1(k) == 0
        Count = Count + 1;
        if k == 64
            Bits = Bits + int32(RLC{33});
            break;
        end
    else
        if Count == 0
            RL = Count;
            Level = abs(x1(k));
            if exist('RLC{RL+1}(Level)','var')
                Bits = Bits + int32(RLC{RL+1}(Level));
            else
                Bits = Bits + int32(RLC{34}) + int32(14);
            end
            Count = 0;
        elseif Count >= 1 && Count <=17
            RL = Count;
            Level = abs(x1(k));
            if exist('RLC{RL+1}(Level)','var')
                Bits = Bits + int32(RLC{RL+1}(Level));
            else
                Bits = Bits + int32(RLC{34}) + int32(14);
            end
            Count = 0;
        else
            RL = Count;
            if exist('RLC{RL+1}','var')
                Bits = Bits + int32(RLC{RL+1});
            else
                Bits = Bits + int32(RLC{34}) + int32(14);
            end
        end
    end
    k = k + 1;
end