function [Qbits,Qsteps] = DWToptimalBits(C,S,wName,R)
% [Qbits,Qsteps] = DWToptimalBits(C,S,wName,R)
% Assign quantizer bits for the L-level 2D DWT
% coefficients using the optimal bit allocation rule.
% Quantizer bits are rounded to nearest intgeres and
% negative bits are assigned zero values.


L = size(S,1) - 2; % number of DWT levels
Coef = cell(3*L+1,1); % cell array to store L-level DWT coefficients
% extract the detail and approximation DWT coefficients
j = 1;
for k = 1:L
    [Coef{j},Coef{j+1},Coef{j+2}] = detcoef2('all',C,S,k);
    j = j+3;
end
Coef{3*L+1} = appcoef2(C,S,wName,L);
% Compute and store the coefficient variances
CoefVar = zeros(3*L+1,1);
for k = 1:L
    k1 = (k-1)*3;
    for j = 1:3
        CoefVar(k1+j) = std2(Coef{k1+j})*std2(Coef{k1+j});
        if CoefVar(k1+j) == 0
            CoefVar(k1+j) = 1;
        end
    end
end
CoefVar(3*L+1) = std2(Coef{3*L+1})*std2(Coef{3*L+1});
% Geometric mean of variances
gm = 1;
p = 1.0/(3*L+1);
for j = 1:L
    j1 = (j-1)*3;
    gm = gm * prod(CoefVar(j1+1:j1+3));
end
gm = (gm*CoefVar(3*L+1))^p;
%
Qbits = zeros(3*L+1,1);
% compute quantizer bits using coefficient variances and
% geometric mean of the variances
for k = 1:3*L+1
    Qbits(k) = round(R + 0.5*log2(CoefVar(k)/gm));
    if Qbits(k) < 0
        Qbits(k) = 0;
    end
end
% Compute the quantization steps
% note that zero quantizer bit is not assigned infinity to Qstep
Qsteps  = zeros(3*L+1,1);
for k = 1:3*L+1
    maxCoef = max(max(Coef{k}));
    D = maxCoef;
    if D ~= 0
        Qsteps(k) = D/(2*2^Qbits(k));
    else
        Qsteps(k) = 1.0e+16;
    end
end