% DisplayDWT.m
% Computes the Two-dimensional Discrete Wavelet Transform 
% of an image up to three levels using user specified wavelet. 
% The DWT coefficients are placed in a large array for display
% purpose. The innermost 4 quadrants correspond to
% LL,HL,LH, and HH coefficients at level 3.
% the three outer quadrants correspond to HL,LH, and HH
% coefficients at levels 1 and 2. 
%
clear
inFile = 'cameraman.tif';
A = imread(inFile);
[Height,Width,Depth] = size(A);
if Depth == 1
    f = double(A);
else
    f1 = rgb2ycbcr(A);
    f = double(f1(:,:,1));
    clear f1
end
wName = 'db2';
L = 3;
% L-level 2D DWT
[DWT,S] = wavedec2(f,L,wName);
% array to store all DWT coefficients for display
S1 = size(S,1);
y = zeros(sum(S(1:S1-1,1)),sum(S(1:S1-1,2)));
% Extract approximation & detail coefficients @ level 2 and
% Place them in the large single array y for display
y(1:S(1,1),1:S(1,2)) = appcoef2(DWT,S,L,wName);
[H,V,D] = detcoef2('all',DWT,S,L);
y(1:S(2,1),S(2,2)+1:2*S(2,2)) = H;
y(S(2,1)+1:2*S(2,1),1:S(2,2)) = V;
y(S(2,1)+1:2*S(2,1),S(2,2)+1:2*S(2,2)) = D;
clear H V D
%
[H,V,D] = detcoef2('all',DWT,S,L-1);
y(1:S(3,1),S(3,2)+1:2*S(3,2)) = H;
y(S(3,1)+1:2*S(3,1),1:S(3,2)) = V;
y(S(3,1)+1:2*S(3,1),S(3,2)+1:2*S(3,2)) = D;
clear H V D
%
[H,V,D] = detcoef2('all',DWT,S,L-2);
y(1:S(4,1),S(4,2)+1:2*S(4,2)) = H;
y(S(4,1)+1:2*S(4,1),1:S(4,2)) = V;
y(S(4,1)+1:2*S(4,1),S(4,2)+1:2*S(4,2)) = D;
% use wcodemat for visual enhancement
figure,imshow(wcodemat(y,128,'m',1),[]) 
title([num2str(L) '-Level ' wName ' ' '2D DWT'])