% ShowPhaseDistortion.m
% Illustration of the effect of non-linear
% phase on the filtered image
% IIR filter with non-linear phase and
% FIR filter with linear phase are considered.
% IIR filter is 9th order and FIR filter has 9 taps.


clear
% design IIR Chebychev filter of order N, stopband
% riplle A dB down, and stopband edge frequency Ef
% Ef should be 0 < Ef < 1
A = 30;
N = 9; 
Ef = 0.5;
[b,a] = cheby2(N,A,Ef);
[H,W] = freqz(b,a,256);
subplot(2,2,1),plot(W,abs(H),'k','LineWidth',2)
title('IIR Magnitude response')
subplot(2,2,2),plot(W,angle(H),'k','LineWidth',2)
title('IIR Phase response')
%
% design a lowpass FIR filter with N taps (N-1-st order)
% with passband edge at Ef
b1 = fir1(N-1,Ef);
a1 = zeros(size(b1));
a1(1) = 1;
[H1,W1] = freqz(b1,a1,256);
subplot(2,2,3),plot(W1,abs(H1),'k','LineWidth',2)
title('FIR Magnitude response')
subplot(2,2,4),plot(W,angle(H1),'k','LineWidth',2)
title('FIR Phase response')
%
% read an image and filter it through IIR & FIR filters
f = imread('cameraman.tif');
%f = imread('birds.ras');
[Height,Width,Depth] = size(f);
if Depth == 3
    f1 = rgb2ycbcr(f);
    f = f1(:,:,1);
    clear f1
end
% filter row by row first
g = zeros(Height,Width);
y = zeros(Height,Width);
for m = 1:Height
g(m,:) = filter(b,a,double(f(m,:)));
end
% next filter along columns
for m = 1:Width
y(:,m) = filter(b,a,double(g(:,m)));
end
% Next, filter the original input image through linear phase FIR
% filter
% The function "filter2" does the 2D filtering
g1 = filter2(b1,f,'same');
% Display the images
figure,imshow(uint8(round(y))),title('Filter with nonlinear phase')
figure, imshow(uint8(round(g1))), title('Filter with linear phase')