function y = quantizeDWT(C,S,Qsteps)
% y = quantizeDWT(C,S,Qsteps)
% quantizes uniformly the L-level DWT coefficients
% using the quantization steps calculated
% from the optimal bit assignment rule.
% Input:
%   C = vectors od DWT coefficients obtained from
%   wavedec2.
%   S = book keeping variables, also obtained from wavedec2.
%   Qsteps = quantization steps corresponding to the
%   optimal bits or integer bits  assignment.
%
% Output:
%   y = quantized/dequantized and inverse discrete wavelet 
%   transformed image.


L = size(S,1) - 2; % number of DWT levels
% extract the DWT coefficients
appCoefLength = S(1,1)*S(1,2); % length of app coef.
for k = 1:L
    k1 = (k-1)*3+1;
    k2 = k1 +2;
    detCoefLength(k1:k2) = S(k+1,1)*S(k+1,2);
end
temp = C(1:appCoefLength);
% quantize the approximation coefficients
temp = floor(temp/Qsteps(3*L+1)+0.5)*Qsteps(3*L+1);
y(1:appCoefLength) = temp;
startLoc = appCoefLength; % length of app. coef. vector
% quantize the detail coefficients
for k = 1:3*L
    temp = C(startLoc+1:startLoc+detCoefLength(k));
    temp = floor(temp/Qsteps(k)+0.5)*Qsteps(k);
    y(startLoc+1:startLoc+detCoefLength(k)) = temp;
    startLoc = startLoc + detCoefLength(k);
end
    