% CompressStereo.m

clear
%leftFile = '20062505L.jpg'; rightFile = '20062505R.jpg';
%leftFile = '20062509L.jpg'; rightFile = '20062509R.jpg';
%leftFile = '20062535L.jpg'; rightFile = '20062535R.jpg';
%leftFile = 'AS12-49-7319HR_left'; 
%rightFile = 'AS12-49-7318HR_right';
fileName = 's41_rocks1_color.jpg'; % martian land
%
%L = imread(leftFile); % left original
%R = imread(rightFile); % right original
A = imread(fileName);
L = A(1:384,1:512,:);
R = A(1:384,1026:1537,:);
clear A
% Make image size divisible by 8
[X,Y,Z] = size(L);
if mod(X,8) ~=0
    Height = floor(X/8)*8;
else
    Height = X;
end
if mod(Y,8) ~=0
    Width = floor(Y/8)*8;
else
    Width = Y;
end
Depth = Z;
clear X Y Z
N = 8; % block size N x N
W = 16; % search window size W x W
figure,imshow(L),title('Original left image')
figure,imshow(R),title('Original right image')
% compress left image using JPEG scheme
%Lq = jpgCompress(L(1:Height,1:Width),'4:2:2');%  for astronaut image
%Lq = jpgCompress(L(1:Height,1:Width,:),'4:2:2');% for japan image
% convert left & right RGB images to YCbCr components
if Depth > 1
    L1 = rgb2ycbcr(L(1:Height,1:Width,:));% uncompressed YCbCr of left image
    Ly = double(L1(:,:,1));% uncompressed Y of left image
    %Lq = rgb2ycbcr(Lq);% compressed YCbCr of left image 
    %Lyq = double(Lq(:,:,1)); % compressed Y component of left image
    R1 = rgb2ycbcr(R(1:Height,1:Width,:));% uncompressed YCbCr of right image
    Ry = double(R1(:,:,1));% uncompressed Y component of right image
else
    Ly = double(L(1:Height,1:Width));% uncompressed left intensity image
    %Lyq = double(Lq); % compressed left intensity image
    Ry = double(R(1:Height,1:Width)); % uncompressed right intensity image
end
% pad left & right intensity images
%Lyq = double(padarray(Lyq,[W/2 W/2],'replicate'));
Ly = double(padarray(Ly,[W/2 W/2],'replicate'));
Ry = double(padarray(Ry,[W/2 W/2],'replicate'));
% do an MC prediction and reconstruction of right image
% from left image
for r = N:N:Height
    rblk = floor(r/N);
    for c = N:N:Width
        cblk = floor(c/N);
        D = 1.0e+10;% initial city block distance
        for u = -N:N
            for v = -N:N
                %d = Ry(r+1:r+N,c+1:c+N)-Lyq(r+u+1:r+u+N,c+v+1:c+v+N);
                d = Ry(r+1:r+N,c+1:c+N)-Ly(r+u+1:r+u+N,c+v+1:c+v+N);
                d = sum(abs(d(:)));% city block distance between pixels
                if d < D
                    D = d;
                    x1 = v; y1 = u; % motion vector
                end
            end
        end
        %temp = Ry(r+1:r+N,c+1:c+N)...
        %        -Lyq(r+1+y1:r+y1+N,c+1+x1:c+x1+N);
        temp = Ry(r+1:r+N,c+1:c+N)...
                -Ly(r+1+y1:r+y1+N,c+1+x1:c+x1+N);
        temp = dct2(temp);
        s = sign(temp); 
        temp = s .* round(abs(temp)/8)*8; % quantize/dequantize DCT
        temp = idct2(temp);
        pf(r-N+1:r,c-N+1:c) = temp;
        %pf(r-N+1:r,c-N+1:c) = Ry(r+1:r+N,c+1:c+N)...
        %        -Lqy(r+1+y1:r+y1+N,c+1+x1:c+x1+N);
        %Ryq(r-N+1:r,c-N+1:c) = Lyq(r+1+y1:r+y1+N,c+1+x1:c+x1+N) + pf(r-N+1:r,c-N+1:c);
        Ryq(r-N+1:r,c-N+1:c) = Ly(r+1+y1:r+y1+N,c+1+x1:c+x1+N) + pf(r-N+1:r,c-N+1:c);
    end
end
% Rq is the quantized YCbCr of right image
if Depth > 1
    Rq(:,:,1) = Ryq; 
    Rq(:,:,2) = R1(1:Height,1:Width,2);
    Rq(:,:,3) = R1(1:Height,1:Width,3);
end
%
figure,imshow(pf,[]), title('DC difference')
%figure,imshow(Lqy(N+1:Height+N,N+1:Width+N)-Ry(N+1:Height+N,N+1:Width+N),[])
figure,imshow(Ly(1:Height,1:Width)-Ry(1:Height,1:Width),[]),title('Simple difference')
figure,imshow(uint8(Ryq)),title('DC reconstructed right image')
[hNoMC,BinNoMC] = hist(Ly(1:Height,1:Width)-Ry(1:Height,1:Width),256);
[hMC,BinMC] = hist(pf,256);
figure,plot(BinMC,hMC,'k'),title('histogram: DC differencing')
xlabel('Pixel difference'), ylabel('Count')
figure,plot(BinNoMC,hNoMC,'k'),title('histogram: simple differencing')
xlabel('Pixel difference'), ylabel('Count')
%
MSE1 = std2(pf)^2;
MSE2 = std2(Ly(1:Height,1:Width) - Ry(1:Height,1:Width))^2;
sprintf('MSE with MC = %7.2f\tMSE without MC = %7.2f\n',MSE1,MSE2)
% Convert quantized right YCbCr back to RGB
%{
if Depth > 1
    figure,imshow(ycbcr2rgb(Lq)), title('Compressed left image')
    figure,imshow(ycbcr2rgb(uint8(round(Rq)))), title('Compressed right image')
end
%}