% Example9_1.m
% Computes the motion vectors and motion compensated
% prediction error image coressponding to two
% temporally adjacent image frames using the
% Sum of Absolute Diference (SAD) metric and
% full search procedure.
% It can estimate the motion to integer pel, half pel, or
% quarter pel accuracy.
% Motion blocks are of size 8 x 8 pixels and the
% search window size is 16 x 16 pixels.


clear
inFile1 = 'tt040.ras';% table tennis sequence
inFile2 = 'tt041.ras';
%inFile1 = 'twy039.ras';% Trevor sequence
%inFile2 = 'twy040.ras';
%inFile2 = 'twy041.ras';
%inFile1 = 'clairey030.yuv'; % Claire sequence
%inFile2 = 'clairey036.yuv';
A = imread(inFile1); % frame #1
B = imread(inFile2); % frame #2
N = 8;% block size is N x N pixels
W = 16; % search window size is W x W pixels
PredictionType = 'full'; % Options: "full", "half" and "quarter"
% Make image size divisible by 8
[X,Y,Z] = size(A);
if mod(X,8) ~=0
    Height = floor(X/8)*8;
else
    Height = X;
end
if mod(Y,8) ~=0
    Width = floor(Y/8)*8;
else
    Width = Y;
end
Depth = Z;
clear X Y Z
%
t1 = cputime; % start CPU time
% call appropriate motion estimation routine
switch PredictionType
    case 'full'
        [x,y,pf] = MCpredict_Full(A,B,N,W);
    case 'half'
        [x,y,pf] = MCpredict_Half(A,B,N,W);
    case 'quarter'
        [x,y,pf] = MCpredict_Quarter(A,B,N,W);
end
t2 = cputime; % end CPU time (t2-t1) is the time for motion estimation
figure,quiver(x,y,'k'),title('Motion Vector')
sprintf('MSE with MC = %3.3f',std2(pf)*std2(pf))
sprintf('MSE without MC = %3.3f',...
    std2(single(A)-single(B))*std2(single(A)-single(B)))
figure,imshow(pf,[]),title('MC prediction')
figure,imshow(single(A)-single(B),[]),title('prediction without MC')
[hNoMC,BinNoMC] = hist(single(A)-single(B),256);
[hMC,BinMC] = hist(pf,256);
figure,plot(BinMC,hMC,'k'),title('histogram of p frame with MC')
figure,plot(BinNoMC,hNoMC,'k')
title('histogram of p frame with no MC')
