% Example9_2.m
% Computes the motion vectors and motion compensated
% prediction error image coressponding to two
% temporally adjacent image frames using the
% Sum of Absolute Diference (SAD) metric and
% Hierarchical search procedure.
% Motion blocks are of size 8 x 8 pixels 
% at the full resolution image.
% Number of levels used is 3. At each higher level,
% the Gaussian lowpass filtered image is subsampled
% by a factor of 2.
% Level 1 is the original or full resolution image,
% level 2 is 1/4 of the original image and
% level 3 image is 1/16 the original image.
% Only integer pel prediction is used in this example.


clear
L = 3; % # levels in the pyramid
P1 = cell(L,1); % cell structure to store the pyramid of reference frame
P2 = cell(L,1); % cell structure to store the pyramid of current frame
inFile1 = 'tt040.ras';% table tennis sequence
inFile2 = 'tt041.ras';
%inFile1 = 'twy039.ras';% Trevor sequence
%inFile2 = 'twy040.ras';
%inFile1 = 'clairey030.yuv'; % Claire sequence
%inFile2 = 'clairey036.yuv';
P1{1} = imread(inFile1); % original image
P2{1} = imread(inFile2);
[Height,Width] = size(P1{1});
% make the image size divisible by 2^(L-1)
if mod(Height,2^(L-1)) ~= 0
    Height = floor(Height/(2^(L-1)))*(2^(L-1));
end
if mod(Width,2^(L-1)) ~= 0
    Width = floor(Width/(2^(L-1)))*(2^(L-1));
end
% Create the L-level pyramid
M = Height; N = Width;
for l = 2:L
    P1{l} = imfilter(P1{l-1},fspecial('gaussian',7,2.5),'symmetric','same');
    P2{l} = imfilter(P2{l-1},fspecial('gaussian',7,2.5),'symmetric','same');
    % Subsample lowpass filtered image by 2 in both dimensions
    P1{l} = P1{l}(1:2:M,1:2:N);
    P2{l} = P2{l}(1:2:M,1:2:N);
    M = M/2; N = N/2;
end
%
figure,subplot(3,2,1),imshow(P1{1}),title('3-level reference frame')
subplot(3,2,3),imshow(P1{2})
subplot(3,2,5),imshow(P1{3})
subplot(3,2,2),imshow(P2{1}),title('3-level current frame')
subplot(3,2,4),imshow(P2{2})
subplot(3,2,6),imshow(P2{3})
t1 = cputime; % start counting CPU time
% Find motion vectors & prediction error using hierarchical search
[x,y,pf] = MCpyramid_Full(P1,P2);
t2 = cputime; % stop counting CPU time; t2-t1 is the time for ME
% quiver plot the MV at level 3
figure,quiver(y{3},x{3},'k'),title(['Motion Vectors at Level ' num2str(3)])
xlabel('Horizontal component')
ylabel('Vertical component')
% quiver plot the MV at level23
figure,quiver(y{2},x{2},'k'),title(['Motion Vectors at Level ' num2str(2)])
xlabel('Horizontal component')
ylabel('Vertical component')
% quiver plot the MV at level 1
figure,quiver(y{1},x{1},'k'),title(['Motion Vectors at Level ' num2str(1)])
xlabel('Horizontal component')
ylabel('Vertical component')
% Display prediction error image
figure,imshow(pf,[]), title('Prediction error image')
% Calculate prediction error variance w/ & without MC
sprintf('MC prediction error variance = %3.3f\n',std2(pf)^2)
sprintf('MSE without MC = %3.3f',...
    std2(single(P2{1})-single(P1{1}))*std2(single(P2{1})-single(P1{1})))
% Plot the histograms of the prediction error w/ & without MC
[hNoMC,BinNoMC] = hist(single(P2{1})-single(P1{1}),256);
[hMC,BinMC] = hist(pf,256);
figure,plot(BinMC,hMC,'k'),title('histogram of p frame with MC')
figure,plot(BinNoMC,hNoMC,'k')
title('histogram of p frame with no MC')