% Example9_3.m
% Video coding using motion compensated prediction
% Block motion is estimated to an integer pel accuracy
% using full search and SAD matching metric.
% Block size is 8 x 8 pixels.
% Search window size is 16 x 16 pixels.
% The differential block is 2D DCT transformed, quantized
% using uniform quantizer with constant quantization step of 16.
% The reconstructed block becomes the reference block for
% the next input frame.
% Only intensity (Luma) image sequence is accepted.
% Note: Both Table Tennis and Trevor sequences have ".ras" 
% extension while the Claire sequence has ".yuv" extension.


clear
inFile1 = 'tt040.ras';% Table Tennis sequence
%inFile1 = 'twy040.ras'; % Trevor sequence
%inFile1 = 'clairey040.yuv'; % Claire sequence
% "strIndx points to the letter "0" in the file name.
strIndx = regexp(inFile1,'0');
A = imread(inFile1); % frame #1
N = 8;% block size is N x N pixels
W = 16; % search window size is W x W pixels
% Make image size divisible by 8
[X,Y,Z] = size(A);
if mod(X,8) ~=0
    Height = floor(X/8)*8;
else
    Height = X;
end
if mod(Y,8) ~=0
    Width = floor(Y/8)*8;
else
    Width = Y;
end
Depth = Z;
clear X Y Z
% pad the initial frame left & right and top & bottom
A = double(padarray(A,[W/2 W/2],'replicate')); % for MC prediction
A1 = double(padarray(A,[W/2 W/2],'replicate')); % for simple differencing
N2 = 2*N;
F = int16(41:49); % number of frames to encode
% Find motion vector by exhaustive search to a single pel accuracy
Cnt = 1;
% arrays to store SNR and PSNR
snr = zeros(length(F),1); snr1 = zeros(length(F),1);
psnr = zeros(length(F),1); psnr1 = zeros(length(F),1);
figure
figure
for f = F
    inFile = strcat(inFile1(1:strIndx(1)),num2str(f),'.ras');
    %inFile = strcat(inFile1(1:strIndx(1)),num2str(f),'.yuv');
    B = imread(inFile); % read the current frame
    figure(1),subplot(3,3,Cnt),imshow(B)
    B = double(padarray(B,[W/2 W/2],'replicate')); % pad the current frame
    for r = N:N:Height
        rblk = floor(r/N);
        for c = N:N:Width
            cblk = floor(c/N);
            D = 1.0e+10;% initial city block distance
            for u = -N:N
                for v = -N:N
                    d = B(r+1:r+N,c+1:c+N)-A(r+u+1:r+u+N,c+v+1:c+v+N);
                    d = sum(abs(d(:)));% city block distance between pixels
                    if d < D
                        D = d;
                        x1 = v; y1 = u; % motion vector
                    end
                end
            end
            % MC compensated difference coding
            temp = B(r+1:r+N,c+1:c+N)...
                -A(r+1+y1:r+y1+N,c+1+x1:c+x1+N);
            TemP = dct2(temp); % DCT of difference
            s = sign(TemP); % extract the coefficient sign
            TemP = s .* round(abs(TemP)/16)*16; % quantize/dequantize DCT
            temp = idct2(TemP); % IDCT
            Br(r-N+1:r,c-N+1:c) = A(r+1+y1:r+y1+N,c+1+x1:c+x1+N) +...
                temp; % reconstructed block
            % simple difference coding
            T = B(r+1:r+N,c+1:c+N)-A1(r+1:r+N,c+1:c+N);
            Tdct = dct2(T); % DCT of differential block
            s = sign(Tdct);
            % quantize/dequantize & IDCT
            Tidct = idct2(s .* round(abs(Tdct)/19)*19);
            % reconstructed block
            Bhat(r-N+1:r,c-N+1:c) = A1(r+1:r+N,c+1:c+N) + Tidct;
        end
    end
    figure(2),subplot(3,3,Cnt),imshow(uint8(round(Br)))
    % Calculate the respective SNRs and PSNRs
    snr(Cnt) = 20*log10(std2(B(N+1:Height+N,N+1:Width+N))/...
        std2(B(N+1:Height+N,N+1:Width+N)-Br));
    snr1(Cnt) = 20*log10(std2(B(N+1:Height+N,N+1:Width+N))/...
        std2(B(N+1:Height+N,N+1:Width+N)-Bhat));
    psnr(Cnt) = 20*log10(255/std2(B(N+1:Height+N,N+1:Width+N)-Br));
    psnr1(Cnt) = 20*log10(255/std2(B(N+1:Height+N,N+1:Width+N)-Bhat));
    % replace previous frames by the currently reconstructed frames
    A = Br;
    A = double(padarray(A,[W/2 W/2],'replicate'));
    A1 = Bhat;
    A1 = double(padarray(A1,[W/2 W/2],'replicate'));
    Cnt = Cnt + 1;
end
figure,plot(F,snr,'k','LineWidth',2), hold on
plot(F,snr1,'k--','LineWidth',2), title('SNR (dB)')
legend('MC','No MC',0)
xlabel('Frame #'), ylabel('SNR (dB)'), hold off
figure,plot(F,psnr,'k','LineWidth',2), hold on
plot(F,psnr1,'k--','LineWidth',2), title('PSNR (dB)')
legend('MC','No MC',0)
xlabel('Frame #'), ylabel('PSNR (dB)'), hold off