% HalfpelPredict_1.m
% computes the motion vector with components x and y
% to 1/2 a pel accuracy 
%
clear
inFile1 = 'tt040.ras';% table tennis sequence
inFile2 = 'tt041.ras';
%inFile1 = 'twy039.ras';% Trevor sequence
%inFile2 = 'twy040.ras';
%inFile2 = 'twy041.ras';
A = imread(inFile1); % frame #1
B = imread(inFile2); % frame #2
%[Height,Width, Depth] = size(A);
N = 8;% block size is N x N pixels
W = 16; % search window size is W x W pixels
[X,Y, Z] = size(A);
if mod(X,N) ~=0
    Height = floor(X/N)*N;
else
    Height = X;
end
if mod(Y,N) ~=0
    Width = floor(Y/N)*N;
else
    Width = Y;
end
Depth = Z;
clear X Y Z
%
A1 = double(padarray(A,[W/2 W/2],'symmetric')); % reference block
B1 = double(padarray(B,[W/2 W/2],'symmetric')); % current block
NumRblk = Height/N;
NumCblk = Width/N;
x = zeros(NumRblk,NumCblk);% x-component of motion vector
y = zeros(NumRblk,NumCblk);% %y-component of motion vector
pf = double(zeros(Height,Width)); % prediction frame
% Find motion vectors to 1/2 pel accuracy
for r = N:N:Height
    rblk = floor(r/N);
    for c = N:N:Width
        cblk = floor(c/N);
        D = 1.0e+10;% initial city block distance
            for u = -N:N
                for l = -N:N
                    RefBlk = A1(r+u+1:r+u+N,c+l+1:c+l+N);
                    %RefBlk = A1(r+u:r+u+N-1,c+l:c+l+N-1);
                    [x2,y2] = meshgrid(r+u+1:r+u+N,c+l+1:c+l+N);
                    %[x2,y2] = meshgrid(r+u:r+u+N-1,c+l:c+l+N-1);
                    [x3,y3] = meshgrid(r+u-0.5:r+u+N-1,c+l-0.5:c+l+N-1);
                    % interpolate at 1/2 pel
                    z1 = interp2(x2,y2,RefBlk,x3,y3,'*linear');
                    CurrentBlk = B1(r+1:r+N,c+1:c+N);
                    %CurrentBlk = B1(r:r+N-1,c:c+N-1);
                    Indx = isnan(z1);
                    z1(Indx == 1) = CurrentBlk(Indx==1);
                    %dd = z1-CurrentBlk;
                    dd = CurrentBlk - z1;
                    d = sum(abs(dd(:)));
                    if d < D
                        D = d;
                        U = u+0.5; L = l+0.5;
                        %U = u-0.5; L = l-0.5;
                        %pf(r:r+N-1,c:c+N-1) = dd;
                        pf(r-N+1:r,c-N+1:c) = dd;
                    end
                end
            end
        x(rblk,cblk) = L; % Motion in the vertical direction  
        y(rblk,cblk) = U; % Motion in the horizontal direction 
    end
end
figure,quiver(x,y,'k'),title('Motion Vector') % plot motion vectors
%
N2 = 2*N;
pfhat = double(zeros(Height,Width)); 
Br = double(zeros(Height,Width));
%
%for r = 1:N:Height
for r = N:N:Height
    %rblk = floor(r/N) + 1;
    rblk = floor(r/N);
    %for c = 1:N:Width
    for c = N:N:Width
        %cblk = floor(c/N)+1;
        cblk = floor(c/N);
        x1 = x(rblk,cblk); y1 = y(rblk,cblk);
        %RefBlk = A1(r+N:r+N2-1,c+N:c+N2-1);
        RefBlk = A1(r+1:r+N,c+1:c+N);
        %[x2,y2] = meshgrid(r+N:r+N2-1,c+N:c+N2-1);
        [x2,y2] = meshgrid(r+1:r+N,c+1:c+N);
        [x3,y3] = meshgrid(r-y1+1:r-y1+N,c-x1+1:c-x1+N);
        %[x3,y3] = meshgrid(r-y1+N:r-y1+N2-1,c-x1+N:c-x1+N2-1);
        z1 = interp2(x2,y2,RefBlk,x3,y3,'*linear');
        %CurrentBlk = B1(r+N:r+N2-1,c+N:c+N2-1);
        CurrentBlk = B1(r+1:r+N,c+1:c+N);
        Indx = isnan(z1);
        %z1(Indx==1) = CurrentBlk(Indx == 1);
        z1(Indx==1) = RefBlk(Indx == 1);
        %pfhat(r:r+N-1,c:c+N-1) = CurrentBlk - z1;
        %Br(r:r+N-1,c:c+N-1)= round(pfhat(r:r+N-1,c:c+N-1) + z1);
        %pfhat(r-N+1:r,c-N+1:c) = CurrentBlk - z1;
        %Br(r-N+1:r,c-N+1:c)= round(pfhat(r-N+1:r,c-N+1:c) + z1);
        Br(r-N+1:r,c-N+1:c)= round(pf(r-N+1:r,c-N+1:c) + z1);
        
    end
end
%
figure,imshow(pf,[]),title('MC prediction')
figure,imshow(uint8(round(Br))),title('Reconstructed image')
figure,imshow(single(A)-single(B),[]),title('prediction without MC')
sprintf('MSE with MC = %3.3f',std2(pf)*std2(pf))
sprintf('MSE without MC = %3.3f',std2(single(A)-single(B))*std2(single(A)-single(B)))
[hNoMC,BinNoMC] = hist(single(A)-single(B),256);
[hMC,BinMC] = hist(pf,256);
figure,plot(BinMC,hMC,'k'),title('histogram of p frame with MC')
figure,plot(BinNoMC,hNoMC,'k')
title('histogram of p frame with no MC')