function [x,y,pf] = MCpyramid_Full(A,B)
% [x,y,pf] = MCpyramid_Full(A,B,N,W)
% Computes motion vectors of N x N moving blocks 
% in an intensity image using full search and
% does a motion compensated prediction to a single pel accuracy
% Input:
%   A = reference frame
%   B = current frame
%   N = block size (nominal value is 8, assumed square)
%   W = search window (2N x 2N)
% Outpur:
%   x = horizontal component of motion vector
%   y = Vertical component of motion vector
%   pf = motion compensated prediction image, same size as input image


[Height,Width] = size(A{3});
% pad input images on left, right, top, and bottom
% padding by replicating works better than padding w/ zeros, which is
% better than symmetric which is better than circular
A1{3} = double(padarray(A{3},[2 2],'replicate'));
B1{3} = double(padarray(B{3},[2 2],'replicate'));
A1{2} = double(padarray(A{2},[4 4],'replicate'));
B1{2} = double(padarray(B{2},[4 4],'replicate'));
A1{1} = double(padarray(A{1},[8 8],'replicate'));
B1{1} = double(padarray(B{1},[8 8],'replicate'));
%
x = cell(3,1); y = cell(3,1);
N = 2;
for L = 3:-1:1
    x{L} = int16(zeros(Height/N,Width/N));
    y{L} = int16(zeros(Height/N,Width/N));
    for r = N:N:Height
        rblk = floor(r/N);
        for c = N:N:Width
            cblk = floor(c/N);
            D = 1.0e+10;% initial city block distance
            for u = -N:N
                for v = -N:N
                    d = B1{L}(r+1:r+N,c+1:c+N)-A1{L}(r+u+1:r+u+N,c+v+1:c+v+N);
                    d = sum(abs(d(:)));% city block distance between pixels
                    if d < D
                        D = d;
                        x{L}(rblk,cblk) = v; y{L}(rblk,cblk) = u;
                    end
                end
            end
        end
    end
    figure,quiver(y{L},x{L},'k'),title(['Level ' num2str(L)])
    N = 2 * N;
    Height = 2 * Height;
    Width = 2 * Width;
end
%
% Reconstruct current frame using prediction error & reference frame
%
N = 8;
[Height,Width] = size(A{1});
N2 = 2*N;
pf = double(zeros(Height,Width)); % prediction frame
Br = double(zeros(Height,Width)); % reconstructed frame
for r = 1:N:Height
    rblk = floor(r/N) + 1;
    for c = 1:N:Width
        cblk = floor(c/N) + 1;
        x1 = x{1}(rblk,cblk); y1 = y{1}(rblk,cblk);
        pf(r:r+N-1,c:c+N-1) = B1{1}(r+N:r+N2-1,c+N:c+N2-1)...
            -A1{1}(r+N+y1:r+y1+N2-1,c+N+x1:c+x1+N2-1);
        Br(r:r+N-1,c:c+N-1) = A1{1}(r+N+y1:r+y1+N2-1,c+N+x1:c+x1+N2-1)...
            + pf(r:r+N-1,c:c+N-1);
    end
end
%
figure,imshow(uint8(round(Br))),title('Reconstructed image')