% TemporalPredict.m
% Computes the difference between two successive
% frames in an image sequence using either
% simple differencing or optimal linear prediction.
% Parameter "DiffType" can be "simple" for simple differencing or
% "optimal" for optimal prediction. Refer to Eq. (9-8)
% for the coefficient value.


clear
%inFile1 = 'tt040.ras';% table tennis sequence
%inFile2 = 'tt041.ras';
%inFile1 = 'twy039.ras';% Trevor sequence
%inFile2 = 'twy040.ras';
inFile1 = 'clairey030.yuv'; % Claire sequence
inFile2 = 'clairey031.yuv';
A = imread(inFile1); % frame #1
B = imread(inFile2); % frame #2
[Height,Width] = size(A);
DiffType = 'optimal'; % options: "simple" and "optimal"
% simple tempral differencing
switch DiffType
    case 'simple'
        pf = double(B) - double(A);
    case 'optimal'
        A1 = double(A)-mean2(double(A)); % remove the mean of ref. frame
        B1 = double(B)-mean2(double(B)); % remove the mean of current frame
        V = sum(sum((double(A)-mean2(double(A))).*...
            (double(A)-mean2(double(A))))); % calculate the sum of 
        % squares of the mean removed reference frame
        a = sum(sum(A1 .* B1))/V;
        pf = double(B) - a*double(A); % calculate the prediction error
end
figure, subplot(2,2,1), imshow(A),title('1st frame')
subplot(2,2,2), imshow(B),title('2nd frame')
sprintf(['MSE with ' DiffType ' temporal differencing = %3.3f'],std2(pf)*std2(pf))
subplot(2,2,3),imshow(pf,[]),title('difference frame')
[hMC,BinMC] = hist(pf,256);
subplot(2,2,4),plot(BinMC,hMC,'k'),title('histogram')